using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Globalization;
//using Gajda.Main;

namespace Gajda.Controls
{
	[DefaultEvent("ValueChanged")]
	public class DoubleTextBox : Gajda.Controls.GajdaTextBox
	{
		private double _x = 0;
    private double _min = Double.NegativeInfinity;
    private double _max = Double.PositiveInfinity;
		private string _format = "";    

		public DoubleTextBox()	{}

		public event EventHandler ValueChanged;

		public override string Text
		{
			get { return base.Text; }
			set
			{
				base.Text = value;
        if (!(value == null) && !Double.TryParse(value.Replace(Constants.BadDecimalSeparator, Constants.DecimalSeparator), NumberStyles.Float, Constants.FormatProvider, out _x))
					_x = Double.NaN;
			}
		}
		[
			Browsable(false),
			DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public virtual double Value
		{
			get{return _x;}
			set
			{
				if (Double.IsNaN(value))
				{
          if (Double.TryParse(base.Text.Replace(Constants.BadDecimalSeparator, Constants.DecimalSeparator), NumberStyles.Float, Constants.FormatProvider, out _x))
						base.Text = "";
					_x = value;
				}
				else
				{
					_x = value;
          base.Text = value.ToString("#.0000E+000", Constants.FormatProvider);
				}
			}
		}
		[DefaultValue(Double.NegativeInfinity)]
		public double Minimum
		{
			get { return _min; }
			set
			{
				_min = value;
				_max = Math.Max(_max, _min);
			}
		}
		[DefaultValue(Double.PositiveInfinity)]
		public double Maximum
		{
			get { return _max; }
			set
			{
				_max = value;
				_min = Math.Min(_max, _min);
			}
		}
		[DefaultValue("")]
		public string Format
		{
			get{return _format;}
			set
			{
				_format = value;
        base.Text = _x.ToString(_format, Constants.FormatProvider);
			}
		}
		[
			Browsable(false),
			DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
		]
		public bool IsEmpty
		{
			get { return Text == ""; }
		}

		protected override void OnLeave(EventArgs e)
		{
			double temp;
      if (!Double.TryParse(base.Text.Replace(Constants.BadDecimalSeparator, Constants.DecimalSeparator), NumberStyles.Float, Constants.FormatProvider, out temp))
			{
				_x = Double.NaN;
				base.Text = "";
			}
			else
			{
				if (_x != temp)
				{
          temp = Math.Max(Math.Min(temp, _max), _min);
					_x = temp;
          base.Text = _x.ToString(_format, Constants.FormatProvider);
					if (ValueChanged != null)
						ValueChanged(this, EventArgs.Empty);
				}
			}
			base.OnLeave(e);
		}
	}
  public static class Constants
  {
    public static string DecimalSeparator;
    public static string BadDecimalSeparator;
    public static CultureInfo FormatProvider = new CultureInfo("en-US");

    static Constants()
    {
      DecimalSeparator = FormatProvider.NumberFormat.NumberDecimalSeparator;
      if (DecimalSeparator == ",")
        BadDecimalSeparator = ".";
      else
        BadDecimalSeparator = ",";
    }
  }

}

