﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CVUT.Vorel;
using CVUT.Vorel.heat.controls;
using System.IO;

namespace CVUT.Vorel.homogenization.controls
{
  public partial class frmHomogenization : Form
  {
    public PrintDocument docToPrint = new PrintDocument();

    public frmHomogenization()
    {
      InitializeComponent();
    }

    private void newToolStripButton_Click(object sender, EventArgs e)
    {
      if (!(saveOption(sender, e) == DialogResult.Cancel))
      {
        if (tabPage1.Visible)
        {
          ucHomoHeat1.newProject(sender, e);
        }
        else if (tabPage2.Visible)
        {
          ucHomoElasticity1.newProject(sender, e);
        }
        else if (tabPage3.Visible)
        {
          ucWeave1.newProject(sender, e);
        }
      }
    }
    private void openToolStripButton_Click(object sender, EventArgs e)
    {
      if (!(saveOption(sender, e) == DialogResult.Cancel))
      {
        if (openFileDialog1.ShowDialog() == DialogResult.OK)
        {
          if (tabPage1.Visible)
          {
            ucHomoHeat1.openProject(sender, e, openFileDialog1.FileName);
          }
          else if (tabPage2.Visible)
          {
            ucHomoElasticity1.openProject(sender, e, openFileDialog1.FileName);
          }
          else if (tabPage3.Visible)
          {
            ucWeave1.openProject(sender, e, openFileDialog1.FileName);
          }

        }
      }
    }
    private void saveToolStripButton_Click(object sender, EventArgs e)
    {
      saveProject(sender, e);
    }
    private void printToolStripButton_Click(object sender, EventArgs e)
    {
      try
      {
        if (tabPage1.Visible)
        {
          ucHomoHeat1.btnPrint_Click(sender, e);
        }
        else if (tabPage2.Visible)
        {
          ucHomoElasticity1.btnPrint_Click(sender, e);
        }
        else if (tabPage3.Visible)
        {
          ucWeave1.btnPrint_Click(sender, e);
        }
      }
      catch (Exception exp)
      {
        System.Console.WriteLine(exp.Message.ToString());
      }
    }
    protected void printPreviewToolStripButton_Click(object sender, System.EventArgs e)
    {
      try
      {
        PrintText _text = new PrintText();
        _text.docToPrint = docToPrint;
        _text.dialForPrintPreview = printPreviewDialog1;

        if (tabPage1.Visible)
        {
          _text.textToPrint = ucHomoHeat1.LogLabel.Text;
        }
        else if (tabPage2.Visible)
        {
          _text.textToPrint = ucHomoElasticity1.LogLabel.Text;
        }
        else if (tabPage3.Visible)
        {
          _text.textToPrint = ucWeave1.LogLabel.Text;
        }

        _text.printPreview(sender, e);
      }
      catch (Exception exp)
      {
        System.Console.WriteLine(exp.Message.ToString());
      }
    }
    private void copyToolStripButton_Click(object sender, EventArgs e)
    {
      if (tabPage1.Visible)
      {
        Clipboard.SetText(ucHomoHeat1.LogLabel.Text);
      }
      else if (tabPage2.Visible)
      {
        Clipboard.SetText(ucHomoElasticity1.LogLabel.Text);
      }
      else if (tabPage3.Visible)
      {
        Clipboard.SetText(ucWeave1.LogLabel.Text);
      }
    }
    private void cutToolStripButton_Click(object sender, EventArgs e)
    {
      DialogResult result;
      result = MessageBox.Show("Do you want to cut the text from the result window?", "Homogenization",
         MessageBoxButtons.OKCancel, MessageBoxIcon.Question);

      if (result == DialogResult.OK)
      {
        if (tabPage1.Visible)
        {
          Clipboard.SetText(ucHomoHeat1.LogLabel.Text);
          ucHomoHeat1.LogLabel.Clear();
        }
        else if (tabPage2.Visible)
        {
          Clipboard.SetText(ucHomoElasticity1.LogLabel.Text);
          ucHomoElasticity1.LogLabel.Clear();
        }
        else if (tabPage3.Visible)
        {
          Clipboard.SetText(ucWeave1.LogLabel.Text);
          ucWeave1.LogLabel.Clear();
        }
      }
    }
    private void calculatorToolStripButton_Click(object sender, EventArgs e)
    {
      CVUT.Vorel.controls.calculator calc = new CVUT.Vorel.controls.calculator();
      calc.Show();
    }
    private void aboutToolStripButton_Click(object sender, EventArgs e)
    {
      CVUT.Vorel.controls.AboutBox about = new CVUT.Vorel.controls.AboutBox();
      about.ShowDialog();
    }
    ///////////////////////////////////////////////////////////////////////////////////
    private DialogResult saveOption(object sender, EventArgs e)
    {
      DialogResult result;
      result = MessageBox.Show("Do you want to save the old project?", "Homogenization",
               MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);

      if (result == DialogResult.Yes)
      {
        result = saveProject(sender, e);
      }
     
      return result;
    }
    private DialogResult saveProject(object sender, EventArgs e)
    {
      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
        if (tabPage1.Visible)
        {
          try
          {
            Save.FileToSave(ucHomoHeat1.Matrix, ucHomoHeat1.InclusionList, ucHomoHeat1.ProjectName, saveFileDialog1.FileName);
            return DialogResult.Yes;
          }
          catch (Exception exc)
          {
            ucHomoHeat1.LogLabel.WriteLine(exc.Message);
            return DialogResult.Cancel;
          }
        }
        else if (tabPage2.Visible)
        {
          try
          {
            Save.FileToSave(ucHomoElasticity1.Matrix, ucHomoElasticity1.InclusionList, ucHomoElasticity1.ProjectName, saveFileDialog1.FileName);
            return DialogResult.Yes;
          }
          catch (Exception exc)
          {
            ucHomoElasticity1.LogLabel.WriteLine(exc.Message);
            return DialogResult.Cancel;
          }
        }
        else if (tabPage3.Visible)
        {
          try
          {
            ucWeave1.saveProject(saveFileDialog1.FileName);
            return DialogResult.Yes;
          }
          catch (Exception exc)
          {
            ucHomoElasticity1.LogLabel.WriteLine(exc.Message);
            return DialogResult.Cancel;
          }
        }
        else
        {
          return DialogResult.Cancel;
        }
      }
      else
      {
        return DialogResult.Cancel;
      }
    }

    private void btnProperties_Click(object sender, EventArgs e)
    {
      frmCopyProperties frmCopyProperties1 = new frmCopyProperties();

      if (frmCopyProperties1.ShowDialog() == DialogResult.OK)
      {
        try
        {
          if (tabPage1.Visible)
          {
            ucWeave1.Change_rbKind(Constants.EProblemKind.Heat);

            switch (frmCopyProperties1.From)
            {
              case Constants.EStuctKind.Matrix:
                if (frmCopyProperties1.To == Constants.EStuctKind.Matrix)
                {
                  ucWeave1.HeatMatrix = ucHomoHeat1.Matrix;
                }
                else if (frmCopyProperties1.To == Constants.EStuctKind.Tow)
                {
                  ucWeave1.LoadTow(ucHomoHeat1.Matrix);
                }
                break;

              case Constants.EStuctKind.Inclusion:
                if (frmCopyProperties1.To == Constants.EStuctKind.Matrix)
                {
                  ucWeave1.LoadMatrix(ucHomoHeat1.SelectedInclusion);
                }
                else if (frmCopyProperties1.To == Constants.EStuctKind.Tow)
                {
                  ucWeave1.LoadTow(ucHomoHeat1.SelectedInclusion);
                }
                break;

              case Constants.EStuctKind.Result:
                heat.Constituent dummy = new heat.Constituent(ucHomoHeat1.Result);

                if (frmCopyProperties1.To == Constants.EStuctKind.Matrix)
                {
                  ucWeave1.LoadMatrix(dummy);
                }
                else if (frmCopyProperties1.To == Constants.EStuctKind.Tow)
                {
                  ucWeave1.LoadTow(dummy);
                }
                break;
            }
          }
          else if (tabPage2.Visible)
          {
            ucWeave1.Change_rbKind(Constants.EProblemKind.Elasticity);

            switch (frmCopyProperties1.From)
            {
              case Constants.EStuctKind.Matrix:
                if (frmCopyProperties1.To == Constants.EStuctKind.Matrix)
                {
                  ucWeave1.ElasticityMatrix = ucHomoElasticity1.Matrix;
                }
                else if (frmCopyProperties1.To == Constants.EStuctKind.Tow)
                {
                  ucWeave1.LoadTow(ucHomoElasticity1.Matrix);
                }
                break;

              case Constants.EStuctKind.Inclusion:
                if (frmCopyProperties1.To == Constants.EStuctKind.Matrix)
                {
                  ucWeave1.LoadMatrix(ucHomoElasticity1.SelectedInclusion);
                }
                else if (frmCopyProperties1.To == Constants.EStuctKind.Tow)
                {
                  ucWeave1.LoadTow(ucHomoElasticity1.SelectedInclusion);
                }
                break;

              case Constants.EStuctKind.Result:
                elasticity.Constituent dummy = new elasticity.Constituent(ucHomoElasticity1.Result);
                if (dummy.EmptyStiffnessMatrix())
                {
                  throw new Exception("Copy Properties: Result: Stiffness matrix is empty!!!");
                }

                if (frmCopyProperties1.To == Constants.EStuctKind.Matrix)
                {
                  ucWeave1.LoadMatrix(dummy);
                }
                else if (frmCopyProperties1.To == Constants.EStuctKind.Tow)
                {
                  ucWeave1.LoadTow(dummy);
                }
                break;
            }
          }

          // visibility of the weave composite tabPage
          tabControl1.SelectedIndex = 2;
        }
        catch(Exception exc)
        {
          if (tabPage1.Visible)
          {
            ucHomoHeat1.LogLabel.WriteLine(exc.Message);
          }
          else if (tabPage2.Visible)
          {
            ucHomoElasticity1.LogLabel.WriteLine(exc.Message);
          }
        }
      }
    }
      

  }
}
