﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CVUT.Vorel.elasticity.controls
{
  public partial class ucHomoElasticity : UserControl
  {
    private List<Inclusion> _inclusions = new List<Inclusion>();
    private Matrix _matrix = new Matrix();
    private Tensor _result = new Tensor(6, 6);
    private string _projectName = "";
    ///////////////////////////////////////////////////////////////////////////////////

    public ucHomoElasticity()
    {
      InitializeComponent();
      ucMaterial1.MaterialConstituent = _matrix;
      ucInclusion1_CiChanged(this, EventArgs.Empty);
      lbInkluze_SelectedIndexChanged(this, EventArgs.Empty);
      tbxProjectName.Text = "New Project";
      cbAnalysis.SelectedIndex = 1;
    }

    public LogLabel LogLabel
    {
      get
      {
        return logLabel;
      }
      set
      {
        logLabel = value;
      }
    }
    [
     Browsable(false),
     DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public List<Inclusion> InclusionList
    {
      get
      {
        return _inclusions;
      }
      set
      {
        _inclusions = value;
      }
    }
    [
     Browsable(false),
     DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public Inclusion SelectedInclusion
    {
      get
      {
        return _inclusions[lbInkluze.SelectedIndex];
      }
    }
    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public Matrix Matrix
    {
      get
      {
        return _matrix;
      }
      set
      {
        _matrix = value;
      }
    }
    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public Tensor Result
    {
      get
      {
        return _result;
      }
    }
    public string ProjectName
    {
      get
      {
        return _projectName;
      }
      set
      {
        _projectName = value;
      }
    }

    private void dtbProjectName_TextChanged(object sender, EventArgs e)
    {
      _projectName = tbxProjectName.Text;
    }
    private void btnAdd_Click(object sender, EventArgs e)
    {
      //if (cbAnalysis.SelectedIndex != 2 || _inclusions.Count < 1)
      _inclusions.Add(new Inclusion());
      _inclusions[_inclusions.Count - 1].name = "new inclusion";
      lbInkluze.Items.Add(_inclusions[_inclusions.Count - 1]);
      lbInkluze.SelectedIndex = _inclusions.Count - 1;

    }
    private void btnRemove_Click(object sender, EventArgs e)
    {
      if (lbInkluze.SelectedIndex > -1)
      {
        int dummy = lbInkluze.SelectedIndex;
        _inclusions.RemoveAt(lbInkluze.SelectedIndex);
        lbInkluze.Items.RemoveAt(lbInkluze.SelectedIndex);
        lbInkluze.SelectedIndex = Math.Min(dummy, _inclusions.Count - 1);
      }
    }
    private void lbInkluze_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (lbInkluze.SelectedIndex > -1)
      {
        ucInclusion1.Enabled = true;
        ucInclusion1.LoadInclusion(_inclusions[lbInkluze.SelectedIndex]);
      }
      else
      {
        ucInclusion1.Enabled = false;
        ucInclusion1.Clear();
      }
    }

    private void ucInclusion1_NameChanged(object sender, EventArgs e)
    {
      int position = lbInkluze.SelectedIndex;
      lbInkluze.Items.Clear();
      for (int i = 0; i < _inclusions.Count; i++)
        lbInkluze.Items.Add(_inclusions[i]);
      lbInkluze.SelectedIndex = position;
    }
    private void ucInclusion1_CiChanged(object sender, EventArgs e)
    {
      double res = 1;
      for (int i = 0; i < _inclusions.Count; i++)
        res -= _inclusions[i].VolFrac;
      lbC0.Text = res.ToString();

      if (res <= 0)
      {
        MessageBox.Show("Volume of inclusions is exceeding 1", "Homogenization", MessageBoxButtons.OK, MessageBoxIcon.Hand);
        lbC0.ForeColor = Color.Red;
        lbC0.Font = new Font(this.Font.Name, (this.Font.SizeInPoints + 5), FontStyle.Bold);
      }
      else if (res < 0.5)
      {
        MessageBox.Show("The computation isn't accurate for this volume of inclusions", "Homogenization", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        lbC0.ForeColor = Color.Blue;
        lbC0.Font = new Font(this.Font.Name, (this.Font.SizeInPoints + 3), FontStyle.Bold);
      }
      else
      {
        lbC0.ForeColor = this.ForeColor;
        lbC0.Font = this.Font;
      }
    }

    private void btnCalculate_Click(object sender, EventArgs e)
    {
      string _methodName = "";

      if (cbAnalysis.SelectedIndex != 2 && cbAnalysis.SelectedIndex != 1 && cbAnalysis.SelectedIndex != 0)
      {
        logLabel.WriteLine("Not yet implemented!!!");
        return;
      }

      try
      {

        switch (cbAnalysis.SelectedIndex)
        {
          case 0:
            _result     = Dilute.Dilute_routine(_matrix, _inclusions, 1.0);
            _methodName = "Dilute"; 
            break;
          case 1:
            _result     = MT.MT_routine(_matrix, _inclusions, 1.0);
            _methodName = "Mori-Tanaka"; 
            break;
          case 2:
            _result = SC.SC_routine(_matrix, _inclusions, 1.0);
            _methodName = "Self-consistent";
            break;
          default:
            throw new Exception("Elasticity: Controls: ucHomoElasticity: btnCalculate_Click: Unsupported method!");
        }

        //output
        logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
        logLabel.WriteLine(_projectName);
        logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
        logLabel.WriteLine("method = " + _methodName);
        logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%");
        logLabel.WriteLine("================= INPUT =================");
        logLabel.WriteLine("--- Matrix ---");
        logLabel.WriteLine("---------------------------");
        _matrix.StiffnessMatrix.InverseMatrix().Print("Compliance matrix", logLabel, false);
        _matrix.StiffnessMatrix.Print("Stiffness matrix", logLabel, false);

        logLabel.WriteLine("--- Inclusion ---");

        for (int i = 0; i < _inclusions.Count; i++)
        {
          logLabel.WriteLine("---------------------------");
          logLabel.WriteLine(_inclusions[i].name);
          logLabel.WriteLine("ci = " + _inclusions[i].VolFrac.ToString());
          _inclusions[i].SemiAxes.Print("Semi-axis", logLabel, false);
          _inclusions[i].EulerAngles.Print("Euler angles", logLabel, false);
          _inclusions[i].StiffnessMatrix.InverseMatrix().Print("Compliance matrix", logLabel, false);
          _inclusions[i].StiffnessMatrix.Print("Stiffness matrix", logLabel, false);
        }
        logLabel.WriteLine("================ OUTPUT ================");
        _result.InverseMatrix().Print("Effective compliance matrix", logLabel, false);
        _result.Print("Effective stiffness matrix", logLabel, false);
      }
      catch (Exception exc)
      {
        logLabel.WriteLine(exc.Message);
      }
    }
    private void btnSave_Click(object sender, EventArgs e)
    {
      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
        using (StreamWriter sw = File.CreateText(saveFileDialog1.FileName))
        {
          sw.Write(logLabel.Text);
        }
      }
    }
    public void btnPrint_Click(object sender, EventArgs e)
    {
      PrintText _text = new PrintText(logLabel.Text, printDialog1, _projectName);

      _text.Print(sender, e);
    }
    private void btnClear_Click(object sender, EventArgs e)
    {
      logLabel.Clear();
    }
    ///////////////////////////////////////////////////////////////////////////////////
    private void clearProject(object sender, EventArgs e)
    {
      _matrix.ClearMatrix();
      ucMaterial1.MaterialConstituent = _matrix;
      _inclusions.Clear();
      ucInclusion1_CiChanged(this, EventArgs.Empty);
      lbInkluze.Items.Clear();
      lbInkluze_SelectedIndexChanged(this, EventArgs.Empty);

      ucInclusion1.Clear();

      tbxProjectName.Text = "New Project";
    }
    public void newProject(object sender, EventArgs e)
    {
      try
      {
        clearProject(sender, e);
      }
      catch (Exception exc)
      {
        logLabel.WriteLine(exc.Message);
      }
    }

    public void openProject(object sender, EventArgs e, string fileName)
    {
      try
      {
        clearProject(sender, e);
        CVUT.Vorel.Load.LoadToProject(out _matrix, out _inclusions, out _projectName, fileName);

        //matrix
        if (_matrix.MaterialKind == Constants.EMaterialKind.FullyAnisotropic)
        {
          throw new Exception("Elasticity: Controls: ucHomoElasticity: openProject: Matrix has fully anisotropy!!!");
        }
        ucMaterial1.MaterialConstituent = _matrix;

        //inclusions
        for (int i = 0; i < _inclusions.Count; i++)
        {
          if (_inclusions[i].MaterialKind == Constants.EMaterialKind.FullyAnisotropic)
          {
            throw new Exception("Elasticity: Controls: ucHomoElasticity: openProject: Inclusion has fully anisotropy!!!");
          }

          lbInkluze.Items.Add(_inclusions[i]);
        }
        lbInkluze.SelectedIndex = _inclusions.Count - 1;

        ucInclusion1_CiChanged(this, EventArgs.Empty);

        tbxProjectName.Text = _projectName;
      }
      catch (Exception exc)
      {
        logLabel.WriteLine(exc.Message);
        clearProject(sender, e);
      }

    }

    private void ucMaterial1_StiffChanged(object sender, EventArgs e)
    {
      _matrix.StiffnessMatrix = ucMaterial1.MaterialConstituent.StiffnessMatrix;
    }

    private void cbAnalysis_SelectedIndexChanged(object sender, EventArgs e)
    {
      if (cbAnalysis.SelectedIndex != 2 && cbAnalysis.SelectedIndex != 1 && cbAnalysis.SelectedIndex != 0)
      {
        logLabel.WriteLine("Not yet implemented!!!");
        return;
      }

      /*if (cbAnalysis.SelectedIndex == 2)
      {
        if (sender.Equals(cbAnalysis))
        {
          logLabel.WriteLine("Only one inclusion supported!!!");
        }
        if (_inclusions.Count > 1)
        {
          for (int i = _inclusions.Count - 1; i > 0; i--)
          {
            lbInkluze.Items.RemoveAt(i);
            _inclusions.RemoveAt(i);
          }
          lbInkluze.SelectedIndex = 0;
        }
      } */
    }

  }
}
