﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CVUT.Vorel.elasticity.controls
{
  public partial class ucMaterial_iso : UserControl
  {
    private Constituent _L  = new Constituent();

    public ucMaterial_iso()
    {
      InitializeComponent();
      cbSource.SelectedIndex = 0;
    }

    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public Constituent Constituent
    {
      get
      {
        return _L;
      }
      set
      {
        LoadL(value);
      }
    }

    private void cbSource_SelectedIndexChanged(object sender, EventArgs e)
    {
      switch (cbSource.SelectedIndex)
      {
        case 0:
          dtbE.Enabled = true;
          dtbNu.Enabled = true;
          dtbG.Enabled = false;
          break;

        case 1:
          dtbE.Enabled = true;
          dtbNu.Enabled = false;
          dtbG.Enabled = true;
          break;

        case 2:
          dtbE.Enabled = false;
          dtbNu.Enabled = true;
          dtbG.Enabled = true;
          break;   
  
        default:
          throw new Exception("Elasticity: controls: ucMaterial_iso: cbSource_SelectedIndexChanged: wrong case!!!");
      }

    }

    private void dtbL_ValueChanged(object sender, EventArgs e)
    {
      switch (cbSource.SelectedIndex)
      {
        case 0:
          dtbG.Value = dtbE.Value / (2 * (1 + dtbNu.Value));
          break;

        case 1:
          dtbNu.Value = dtbE.Value / (2 * dtbG.Value) - 1;
          break;

        case 2:
          dtbE.Value = 2 * dtbG.Value * (1 + dtbNu.Value);
          break;

        default:
          throw new Exception("Elasticity: controls: ucMaterial_iso: dtbL_ValueChanged: wrong case!!!");

      }

      _L.StiffnessMatrix = Tensor.StiffnessMatrix(dtbE.Value, dtbNu.Value);
      StiffChanged(this, EventArgs.Empty);
    }

    public void LoadL(Constituent L)
    {
      Tensor constants = new Tensor(2);

      if (L.MaterialKind == Constants.EMaterialKind.Isotropic)
      {
        _L = L;

        constants = _L.ConstantsFromStiffnessMatrix();
      }
      else if (L.MaterialKind == Constants.EMaterialKind.Orthotropic ||
               L.MaterialKind == Constants.EMaterialKind.Anisotropic)
      {
        Tensor L_help = new Tensor(3, 3);

        for (int i = 0; i < 3; i++)
        {
          L_help[i, i] = L.StiffnessMatrix[0, 0];
          for (int j = i + 1; j < 3; j++)
            L_help[i, j] = L_help[j, i] = L.StiffnessMatrix[0, 1];
        }

        try
        {
          L_help = L_help.InverseMatrix();

          constants[0] = 1 / L_help[0, 0];  // E
          constants[1] = -L_help[0, 1] / L_help[0, 0];  // nu

          _L.StiffnessMatrix = Tensor.StiffnessMatrix(constants[0], constants[1]);
        }
        catch
        {
          throw new Exception("Elasticity: controls: ucMaterial_iso: LoadL: Singular stiffness matrix");
        }

      }
      else
      {
        throw new Exception("Elasticity: Controls: ucMaterial_ortho: LoadL: wrong StiffnessMatrix!!!");
      }

      dtbE.Value = constants[0];
      dtbNu.Value = constants[1];
      dtbG.Value = dtbE.Value / (2 * (1 + dtbNu.Value));

    }

    public void Clear()
    {
      dtbE.Clear();
      dtbNu.Clear();
      dtbG.Clear();
    }

    public event EventHandler StiffChanged;
  }
}
