﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CVUT.Vorel.elasticity.controls
{
  public partial class ucMaterial_ortho : UserControl
  {
    Constituent _L = new Constituent();

    public ucMaterial_ortho()
    {
      InitializeComponent();
    }

    [
     Browsable(false),
     DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public Constituent Constituent
    {
      get
      {
        return _L;
      }
      set
      {
        LoadL(value);
      }
    }

    private void dtbL_ValueChanged(object sender, EventArgs e)
    {
      double nu12, nu13, nu23; //DLE BIBLE (NUMERICKE METODY MECHANIKY I) - STR. 18
                               

      nu12 = dtbnu12.Value; 
      nu13 = dtbnu13.Value; 
      nu23 = dtbnu23.Value; 

      _L.StiffnessMatrix = Tensor.StiffnessMatrix(dtbE11.Value, dtbE22.Value, dtbE33.Value, 
                                                  nu12, nu13, nu23, dtbG23.Value, dtbG31.Value, dtbG12.Value);

      StiffChanged(this, EventArgs.Empty);
    }

    public void LoadL(Constituent L)
    {

      switch (L.MaterialKind)
      {
        case Constants.EMaterialKind.Isotropic:
          {
            Tensor constants = new Tensor(2);

            _L = L;

            constants = _L.ConstantsFromStiffnessMatrix();

            dtbE11.Value = dtbE22.Value = dtbE33.Value = constants[0];
            dtbnu12.Value = dtbnu13.Value = dtbnu23.Value = constants[1];
            dtbG23.Value = dtbG31.Value = dtbG12.Value = constants[0] / (2 * (1 + constants[1]));

            break;
          }

        case Constants.EMaterialKind.Orthotropic:
          {
            Tensor constants = new Tensor(9);

            _L = L;

            constants = _L.ConstantsFromStiffnessMatrix();

            dtbE11.Value = constants[0];
            dtbE22.Value = constants[1];
            dtbE33.Value = constants[2];

            dtbnu12.Value = constants[3];
            dtbnu13.Value = constants[4];
            dtbnu23.Value = constants[5];

            dtbG23.Value = constants[6];
            dtbG31.Value = constants[7];
            dtbG12.Value = constants[8];

            break;
          }
        case Constants.EMaterialKind.Anisotropic:
          {
            Tensor L_help = new Tensor(6, 6);
            Tensor constants = new Tensor(9);

            for (int i = 0; i < 3; i++)
            {
              L_help[i + 3, i + 3] = L.StiffnessMatrix[i + 3, i + 3];
              for (int j = 0; j < 3; j++)
                L_help[i, j] = L.StiffnessMatrix[i, j];
            }

            _L.StiffnessMatrix = L_help;

            constants = _L.ConstantsFromStiffnessMatrix();

            dtbE11.Value = constants[0];
            dtbE22.Value = constants[1];
            dtbE33.Value = constants[2];

            dtbnu12.Value = constants[3];
            dtbnu13.Value = constants[4];
            dtbnu23.Value = constants[5];

            dtbG23.Value = constants[6];
            dtbG31.Value = constants[7];
            dtbG12.Value = constants[8];

            break;
          }

        default:
          throw new Exception("Elasticity: Controls: ucMaterial_ortho: LoadL: wrong StiffnessMatrix!!!");
      }

    }

    public void Clear()
    {
      dtbE11.Clear(); 
      dtbE22.Clear(); 
      dtbE33.Clear(); 

      dtbnu12.Clear();
      dtbnu13.Clear();
      dtbnu23.Clear();

      dtbG23.Clear(); 
      dtbG31.Clear(); 
      dtbG12.Clear();
    }

    public event EventHandler StiffChanged;

  }
}
