﻿using System;
using System.Collections.Generic;
using System.Text;

namespace CVUT.Vorel.elasticity
{
  public class Constituent
  {
    private Tensor _StiffnessMatrix;

    /////////////////////////////////////////////////////////////////////////////////////////////
    public Constituent()
    {
      _StiffnessMatrix = new Tensor(6, 6);

      _StiffnessMatrix[0, 0] = _StiffnessMatrix[1, 1] = _StiffnessMatrix[2, 2] = 1;   // E = 1, nu = 0, G = 0.5
      _StiffnessMatrix[3, 3] = _StiffnessMatrix[4, 4] = _StiffnessMatrix[5, 5] = 0.5;
    }
    public Constituent(double E, double nu)
    {
      _StiffnessMatrix = new Tensor(6, 6);

      _StiffnessMatrix = Tensor.StiffnessMatrix(E, nu);
    }
    public Constituent(Tensor StiffMatrix)
    {
      if (StiffMatrix.which_order != 2 || StiffMatrix.how_many_lines != 6 || StiffMatrix.how_many_columns != 6)
      {
        throw new Exception("Constituent: Constituent(Tensor StiffMatrix): wrong tensor!");
      }

      _StiffnessMatrix = new Tensor(StiffMatrix);
    }
    public Constituent(Constituent oa)
      : this(oa._StiffnessMatrix)
    {
    }
    /////////////////////////////////////////////////////////////////////////////////////////////
    public void ClearConstituent()
    {
      this._StiffnessMatrix.Clear();

      _StiffnessMatrix[0, 0] = _StiffnessMatrix[1, 1] = _StiffnessMatrix[2, 2] = 1;   // E = 1, nu = 0, G = 0.5
      _StiffnessMatrix[3, 3] = _StiffnessMatrix[4, 4] = _StiffnessMatrix[5, 5] = 0.5;
    }

    public Tensor StiffnessMatrix
    {
      get
      {
        return (_StiffnessMatrix);
      }
      set
      {
        _StiffnessMatrix = value;
      }
    }
    public Constants.EMaterialKind MaterialKind
    {
      get
      {
        double err;
        err = Constants.SMALL * Tensor.MinAbsNonZeroElement(_StiffnessMatrix);

        if (Math.Abs(_StiffnessMatrix[0, 1] - _StiffnessMatrix[1, 0]) > err || Math.Abs(_StiffnessMatrix[0, 2] - _StiffnessMatrix[2, 0]) > err || Math.Abs(_StiffnessMatrix[0, 3] - _StiffnessMatrix[3, 0]) > err ||
            Math.Abs(_StiffnessMatrix[0, 4] - _StiffnessMatrix[4, 0]) > err || Math.Abs(_StiffnessMatrix[0, 5] - _StiffnessMatrix[5, 0]) > err || Math.Abs(_StiffnessMatrix[1, 2] - _StiffnessMatrix[2, 1]) > err ||
            Math.Abs(_StiffnessMatrix[1, 3] - _StiffnessMatrix[3, 1]) > err || Math.Abs(_StiffnessMatrix[1, 4] - _StiffnessMatrix[4, 1]) > err || Math.Abs(_StiffnessMatrix[1, 5] - _StiffnessMatrix[5, 1]) > err ||
            Math.Abs(_StiffnessMatrix[2, 3] - _StiffnessMatrix[3, 2]) > err || Math.Abs(_StiffnessMatrix[2, 4] - _StiffnessMatrix[4, 2]) > err || Math.Abs(_StiffnessMatrix[2, 5] - _StiffnessMatrix[5, 2]) > err ||
            Math.Abs(_StiffnessMatrix[3, 4] - _StiffnessMatrix[4, 3]) > err || Math.Abs(_StiffnessMatrix[3, 5] - _StiffnessMatrix[5, 3]) > err || Math.Abs(_StiffnessMatrix[4, 5] - _StiffnessMatrix[5, 4]) > err)
        {
          return (Constants.EMaterialKind.FullyAnisotropic);
        }
        else if (Math.Abs(_StiffnessMatrix[0, 3]) > err || Math.Abs(_StiffnessMatrix[0, 4]) > err || Math.Abs(_StiffnessMatrix[0, 5]) > err ||
                 Math.Abs(_StiffnessMatrix[1, 3]) > err || Math.Abs(_StiffnessMatrix[1, 4]) > err || Math.Abs(_StiffnessMatrix[1, 5]) > err ||
                 Math.Abs(_StiffnessMatrix[2, 3]) > err || Math.Abs(_StiffnessMatrix[2, 4]) > err || Math.Abs(_StiffnessMatrix[2, 5]) > err ||
                 Math.Abs(_StiffnessMatrix[3, 4]) > err || Math.Abs(_StiffnessMatrix[3, 5]) > err || Math.Abs(_StiffnessMatrix[4, 5]) > err)
        {
          return (Constants.EMaterialKind.Anisotropic);
        }
        else if (Math.Abs(_StiffnessMatrix[0, 1] - _StiffnessMatrix[0, 2]) > err || Math.Abs(_StiffnessMatrix[0, 1] - _StiffnessMatrix[1, 2]) > err ||
                 Math.Abs(_StiffnessMatrix[0, 0] - _StiffnessMatrix[1, 1]) > err || Math.Abs(_StiffnessMatrix[0, 0] - _StiffnessMatrix[2, 2]) > err ||
                 Math.Abs(_StiffnessMatrix[3, 3] - _StiffnessMatrix[4, 4]) > err || Math.Abs(_StiffnessMatrix[3, 3] - _StiffnessMatrix[5, 5]) > err)
        {
          return (Constants.EMaterialKind.Orthotropic);
        }
        else
        {
          Tensor M = new Tensor(_StiffnessMatrix.InverseMatrix());

          double E, nu, G;
          E = 1 / M[0, 0];  // E
          nu = -M[0, 1] / M[1, 1];  // nu
          G = E / (2 * (1 + nu));

          if (Math.Abs(_StiffnessMatrix[3, 3] - G) > err)
          {
            return (Constants.EMaterialKind.Orthotropic);
          }
          else
          {
            return (Constants.EMaterialKind.Isotropic);
          }
        }
      }
    }

    public Tensor ConstantsFromStiffnessMatrix()
    {
      try
      {
        switch (this.MaterialKind)
        {
          case Constants.EMaterialKind.Isotropic:
            {
              Tensor result = new Tensor(2);
              Tensor M = new Tensor(_StiffnessMatrix.InverseMatrix());

              result[0] = 1 / M[0, 0];  // E
              result[1] = -M[0, 1] / M[1, 1];  // nu

              return (result);
            }

          case Constants.EMaterialKind.Orthotropic:
            {
              Tensor result = new Tensor(9);
              Tensor M = new Tensor(_StiffnessMatrix.InverseMatrix());

              result[0] = 1 / M[0, 0];    // E11
              result[1] = 1 / M[1, 1];    // E22
              result[2] = 1 / M[2, 2];    // E33

              result[3] = -M[0, 1] / M[1, 1];    // nu12
              result[4] = -M[0, 2] / M[2, 2];    // nu13
              result[5] = -M[1, 2] / M[2, 2];    // nu23

              result[6] = 1 / M[3, 3];    // G23
              result[7] = 1 / M[4, 4];    // G13
              result[8] = 1 / M[5, 5];    // G12

              return (result);
            }

          default:
            throw new Exception("Tensor: GetE_nuFromStiffnessMatrix: stiffness matrix is anisotropic");
        }
      }
      catch (Exception exc)
      {
        throw new Exception(exc.Message);
      }
    }
    public bool EmptyStiffnessMatrix()
    {
      int dummy = 0;
      for (int i = 0; i < 6; i++)
      {
        for (int j = 0; j < 6; j++)
        {
          if (_StiffnessMatrix[i, j] == 0)
              dummy++;
          else
            return false;
        }
      }
        
      if (dummy == 36)
        return true;

      return false;
    }
  }
}
