﻿using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Xml;
using System.Text;

namespace CVUT.Vorel.elasticity
{
  public class Tow : Inclusion
  {

    private double _a_size, _b_size, _g_size, _h_size;
    private DataXY _histogram = new DataXY();
    /////////////////////////////////////////////////////////////////////////////////////////////
    public Tow()
      : base()
    {
      _a_size = _b_size = _g_size = _h_size = 0.0;

    }
    public Tow(double A, double B, double G, double H, Inclusion oa)
      : base(oa)
    {
      _a_size = A;
      _b_size = B;
      _g_size = G;
      _h_size = H;
    }
    public Tow(Tow oa)
      : base(oa)
    {
    }
    /////////////////////////////////////////////////////////////////////////////////////////////
    public double a
    {
      get
      {
        return (_a_size);
      }
      set
      {
        _a_size = value;
      }
    }
    public double b
    {
      get
      {
        return (_b_size);
      }
      set
      {
        _b_size = value;
      }
    }
    public double g
    {
      get
      {
        return (_g_size);
      }
      set
      {
        _g_size = value;
      }
    }
    public double h
    {
      get
      {
        return (_h_size);
      }
      set
      {
        _h_size = value;
      }
    }
    public DataXY histogram
    {
      get
      {
        return _histogram;
      }
      set
      {
        _histogram = value;
      }
    }


    public void ClearTow()
    {
      base.ClearInclusion();
      _a_size = _b_size = _g_size = _h_size = 0.0;
    }

    public XmlElement ToXmlh(XmlDocument xmldoc)
    {
      XmlElement _tow;
      XmlElement _text1;
      XmlElement _text2;

      _tow = xmldoc.CreateElement("tow");

      //unit cell
      _text1 = xmldoc.CreateElement("unit_cell");
      _text2 = xmldoc.CreateElement("a");
      _text2.InnerText = Convert.ToString(this.a, Constants.Separators.FormatProvider);
      _text1.AppendChild(_text2);

      _text2 = xmldoc.CreateElement("b");
      _text2.InnerText = Convert.ToString(this.b, Constants.Separators.FormatProvider);
      _text1.AppendChild(_text2);

      _text2 = xmldoc.CreateElement("g");
      _text2.InnerText = Convert.ToString(this.g, Constants.Separators.FormatProvider);
      _text1.AppendChild(_text2);

      _text2 = xmldoc.CreateElement("h");
      _text2.InnerText = Convert.ToString(this.h, Constants.Separators.FormatProvider);
      _text1.AppendChild(_text2);

      _tow.AppendChild(_text1);

      //volume fraction
      _text1 = xmldoc.CreateElement("vol_frac");
      _text1.InnerText = Convert.ToString(VolFrac, Constants.Separators.FormatProvider);
      _tow.AppendChild(_text1);

      //semi axes
      _tow.AppendChild(SemiAxes.ToXml(xmldoc, "semi_axes"));

      _tow.AppendChild(EulerAngles.ToXml(xmldoc, "Euler_angles"));

      _tow.AppendChild(StiffnessMatrix.ToXml(xmldoc, "stiffness_matrix"));

      return (_tow);
    }
    public void FromXmlh(XmlNode xmlnode)
    {
      string name = "";
      this.ClearTow();
      int[] count = new int[5];

      count[0] = count[1] = count[2] = count[3] = count[4] = 0;

      this.name = "";
      for (int i = 0; i < xmlnode.Attributes.Count; i++)
      {
        if (xmlnode.Attributes[i].Name == "name")
        {
          this.name = xmlnode.Attributes[i].Value;
        }
      }

      if (!(xmlnode.ChildNodes.Count == 5))
      {
        throw new Exception("xml: inclusion: wrong number of children!!!");
      }
      else
      {
        for (int i = 0; i < 5; i++)
        {
          switch (xmlnode.ChildNodes[i].Name)
          {
            case "unit_cell":
              count[4]++;

              if (!(xmlnode.ChildNodes[i].ChildNodes.Count == 4))
              {
                throw new Exception("xml: tow: wrong unit_cell!!!");
              }
              for (int j = 0; j < 4; j++)
              {
                switch (xmlnode.ChildNodes[i].ChildNodes[j].Name)
                {
                  case "a":
                    _a_size = Functions.StringToDouble(xmlnode.ChildNodes[i].ChildNodes[j].InnerText, "xml: tow: wrong a!!!");
                    break;
                  case "b":
                    _b_size = Functions.StringToDouble(xmlnode.ChildNodes[i].ChildNodes[j].InnerText, "xml: tow: wrong b!!!");
                    break;
                  case "g":
                    _g_size = Functions.StringToDouble(xmlnode.ChildNodes[i].ChildNodes[j].InnerText, "xml: tow: wrong g!!!");
                    break;
                  case "h":
                    _h_size = Functions.StringToDouble(xmlnode.ChildNodes[i].ChildNodes[j].InnerText, "xml: tow: wrong h!!!");
                    break;
                  default:
                    throw new Exception("xml: tow: wrong tensor name!!!");
                }

              }
              break;

            case "vol_frac":
              count[0]++;
              VolFrac = Functions.StringToDouble(xmlnode.ChildNodes[i].InnerText, "xml: tow: wrong vol_frac!!!");
              break;

            case "tensor":
              for (int j = 0; j < xmlnode.ChildNodes[i].Attributes.Count; j++)
              {
                if (xmlnode.ChildNodes[i].Attributes[j].Name == "name")
                {
                  name = xmlnode.ChildNodes[i].Attributes[j].Value;
                }
              }
              switch (name)
              {
                case "semi_axes":
                  count[1]++;
                  this.SemiAxes.FromXml(xmlnode.ChildNodes[i]);
                  break;

                case "Euler_angles":
                  count[2]++;
                  this.EulerAngles.FromXml(xmlnode.ChildNodes[i]);
                  break;

                case "stiffness_matrix":
                  count[3]++;
                  this.StiffnessMatrix.FromXml(xmlnode.ChildNodes[i]);
                  break;

                default:
                  throw new Exception("xml: tow: wrong tensor name!!!");
              }
              break;

            default:
              throw new Exception("xml: tow: wrong tensor child!!!");
          }

        }
        if (!(count[0] == 1 && count[1] == 1 && count[2] == 1 && count[3] == 1 && count[4] == 1))
        {
          throw new Exception("xml: tow: there miss some tow parameters!!!");
        }

      }
    }



  }
}
