﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Text;
using System.IO;

namespace CVUT.Vorel
{
  class Load
  {
    private static XmlDocument _xmldoc;

    public static Constants.EProblemKind WhichCase(string fileName)
    {
      XmlElement _root;
      FileStream fs;
      Constants.EProblemKind type = Constants.EProblemKind.Empty;

      fs = new FileStream(fileName, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
      _xmldoc = new XmlDocument();
      _xmldoc.Load(fs);
      fs.Close();

      //project name
      _root = _xmldoc.DocumentElement;
      if (!(_root.Name == "homogenization"))
      {
        throw new Exception("xml: load: WhichCase: wrong root name!!!");
      }
      for (int i = 0; i < _root.Attributes.Count; i++)
      {
        if (_root.Attributes[i].Name == "type")
        {
          switch(_root.Attributes[i].Value)
          {
            case "elasticity":
              type = Constants.EProblemKind.Elasticity;
              break;
            case "heat":
              type = Constants.EProblemKind.Heat;
              break;
          }
        }
      }

      return type;
    }
    public static void LoadToProject(out heat.Matrix matrix, out List<heat.Inclusion> incl_list, out string projectname, string filename)
    {
      XmlElement _root;
      FileStream fs;
      projectname = "New Project";
      matrix = new heat.Matrix();
      incl_list = new List<heat.Inclusion>();

      int[] count = new int[2];
      count[0] = count[1] = 0;

      fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
      _xmldoc = new XmlDocument();
      _xmldoc.Load(fs);
      fs.Close();

      //project name
      _root = _xmldoc.DocumentElement;
      if (!(_root.Name == "homogenization"))
      {
        throw new Exception("xml: load: wrong root name!!!");
      }
      for (int i = 0; i < _root.Attributes.Count; i++)
      {
        if (_root.Attributes[i].Name == "name")
        {
          projectname = _root.Attributes[i].Value;
        }
      }

      if (!(_root.ChildNodes.Count == 2))
      {
        throw new Exception("xml: load: too many children in root!!!");
      }
      for (int i = 0; i < 2; i++)
      {
        if (_root.ChildNodes[i].Name == "matrix")
        {
          count[0]++;
          matrix.FromXml(_root.ChildNodes[i]);
        }
        else if (_root.ChildNodes[i].Name == "inclusions")
        {
          count[1]++;
          for (int j = 0; j < _root.ChildNodes[i].ChildNodes.Count; j++)
          {
            incl_list.Add(new heat.Inclusion());
            incl_list[j].FromXml(_root.ChildNodes[i].ChildNodes[j]);
          }
        }
        else
        {
          throw new Exception("xml: load: wrong child in root!!!");
        }
      }

      if (!(count[0] == 1 && count[1] == 1))
      {
        throw new Exception("xml: load: there miss some children!!!");
      }
    }
    public static void LoadToProject(out elasticity.Matrix matrix, out List<elasticity.Inclusion> incl_list, out string projectname, string filename)
    {
      XmlElement _root;
      FileStream fs;
      projectname = "New Project";
      matrix = new elasticity.Matrix();
      incl_list = new List<elasticity.Inclusion>();

      int[] count = new int[2];
      count[0] = count[1] = 0;

      fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
      _xmldoc = new XmlDocument();
      _xmldoc.Load(fs);
      fs.Close();

      //project name
      _root = _xmldoc.DocumentElement;
      if (!(_root.Name == "homogenization"))
      {
        throw new Exception("xml: load: wrong root name!!!");
      }
      for (int i = 0; i < _root.Attributes.Count; i++)
      {
        if (_root.Attributes[i].Name == "name")
        {
          projectname = _root.Attributes[i].Value;
        }
      }

      if (!(_root.ChildNodes.Count == 2))
      {
        throw new Exception("xml: load: too many children in root!!!");
      }
      for (int i = 0; i < 2; i++)
      {
        if (_root.ChildNodes[i].Name == "matrix")
        {
          count[0]++;
          matrix.FromXml(_root.ChildNodes[i]);
        }
        else if (_root.ChildNodes[i].Name == "inclusions")
        {
          count[1]++;
          for (int j = 0; j < _root.ChildNodes[i].ChildNodes.Count; j++)
          {
            incl_list.Add(new elasticity.Inclusion());
            incl_list[j].FromXml(_root.ChildNodes[i].ChildNodes[j]);
          }
        }
        else
        {
          throw new Exception("xml: load: wrong child in root!!!");
        }
      }

      if (!(count[0] == 1 && count[1] == 1))
      {
        throw new Exception("xml: load: there miss some children!!!");
      }
    }

    public static void LoadToProject(out elasticity.Matrix matrix, out elasticity.Tow tow, out string projectname, string filename)
    {
      XmlElement _root;
      FileStream fs;
      projectname = "New Project";
      matrix = new elasticity.Matrix();
      tow = new elasticity.Tow();

      int[] count = new int[2];
      count[0] = count[1] = 0;

      fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
      _xmldoc = new XmlDocument();
      _xmldoc.Load(fs);
      fs.Close();

      //project name
      _root = _xmldoc.DocumentElement;
      if (!(_root.Name == "homogenization"))
      {
        throw new Exception("xml: load: wrong root name!!!");
      }
      for (int i = 0; i < _root.Attributes.Count; i++)
      {
        if (_root.Attributes[i].Name == "name")
        {
          projectname = _root.Attributes[i].Value;
        }
      }

      if (!(_root.ChildNodes.Count == 2))
      {
        throw new Exception("xml: load: too many children in root!!!");
      }
      for (int i = 0; i < 2; i++)
      {
        if (_root.ChildNodes[i].Name == "matrix")
        {
          count[0]++;
          matrix.FromXml(_root.ChildNodes[i]);
        }
        else if (_root.ChildNodes[i].Name == "tow")
        {
          count[1]++;
          tow.FromXmlh(_root.ChildNodes[i]);
        }
        else
        {
          throw new Exception("xml: load: wrong child in root!!!");
        }
      }

      if (!(count[0] == 1 && count[1] == 1))
      {
        throw new Exception("xml: load: there miss some children!!!");
      }
    }
    public static void LoadToProject(out heat.Matrix matrix, out heat.Tow tow, out string projectname, string filename)
    {
      XmlElement _root;
      FileStream fs;
      projectname = "New Project";
      matrix = new heat.Matrix();
      tow = new heat.Tow();

      int[] count = new int[2];
      count[0] = count[1] = 0;

      fs = new FileStream(filename, FileMode.Open, FileAccess.Read, FileShare.ReadWrite);
      _xmldoc = new XmlDocument();
      _xmldoc.Load(fs);
      fs.Close();

      //project name
      _root = _xmldoc.DocumentElement;
      if (!(_root.Name == "homogenization"))
      {
        throw new Exception("xml: load: wrong root name!!!");
      }
      for (int i = 0; i < _root.Attributes.Count; i++)
      {
        if (_root.Attributes[i].Name == "name")
        {
          projectname = _root.Attributes[i].Value;
        }
      }

      if (!(_root.ChildNodes.Count == 2))
      {
        throw new Exception("xml: load: too many children in root!!!");
      }
      for (int i = 0; i < 2; i++)
      {
        if (_root.ChildNodes[i].Name == "matrix")
        {
          count[0]++;
          matrix.FromXml(_root.ChildNodes[i]);
        }
        else if (_root.ChildNodes[i].Name == "tow")
        {
          count[1]++;
          tow.FromXmlh(_root.ChildNodes[i]);
        }
        else
        {
          throw new Exception("xml: load: wrong child in root!!!");
        }
      }

      if (!(count[0] == 1 && count[1] == 1))
      {
        throw new Exception("xml: load: there miss some children!!!");
      }
    }

  }
}
