﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace CVUT.Vorel.heat.controls
{
  public partial class ucThermo : UserControl
  {
    private Inclusion _inclusion;
    ///////////////////////////////////////////////////////////////////////////////////
    public ucThermo()
    {
      InitializeComponent();
    }

    private void rbSphere_CheckedChanged(object sender, EventArgs e)
    {
      Constants.EInclusionKind dummy = Constants.EInclusionKind.Empty;

      if (rbSphere.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = false;
        dtbA3.Enabled = false;

        dummy = Constants.EInclusionKind.Sphere;
      }
      else if (rbCylinder.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = false;
        dtbA3.Enabled = false;

        dummy = Constants.EInclusionKind.Cylinder;
      }
      else if (rbEllipticCylinder.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = true;
        dtbA3.Enabled = false;

        dummy = Constants.EInclusionKind.EllipticCylinder;
      }
      else if (rbPennyShape.Checked)
      {
        dtbA1.Enabled = false;
        dtbA2.Enabled = false;
        dtbA3.Enabled = true;

        if (dtbA3.Value > 1)
          dtbA3.Value = 1;

        dummy = Constants.EInclusionKind.PennyShape;
      }
      else if (rbEllipsoid.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = true;
        dtbA3.Enabled = true;

        dummy = Constants.EInclusionKind.Ellipsoid;
      }

      switch (dummy)
      {
        case Constants.EInclusionKind.Sphere:
          dtbA3.Value = dtbA2.Value = dtbA1.Value;
          break;
        case Constants.EInclusionKind.Cylinder:
          dtbA2.Value = dtbA1.Value;
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.PennyShape:
          dtbA1.Value = dtbA2.Value = 1;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          break;
      }

      _inclusion.SemiAxes[0] = dtbA1.Value;
      _inclusion.SemiAxes[1] = dtbA2.Value;
      _inclusion.SemiAxes[2] = dtbA3.Value;
    }
    private void dtbA_ValueChanged(object sender, EventArgs e)
    {
      Constants.EInclusionKind dummy = Constants.EInclusionKind.Empty;

      if (rbSphere.Checked)
      {
        dummy = Constants.EInclusionKind.Sphere;
      }
      else if (rbCylinder.Checked)
      {
        dummy = Constants.EInclusionKind.Cylinder;
      }
      else if (rbEllipticCylinder.Checked)
      {
        dummy = Constants.EInclusionKind.EllipticCylinder;
      }
      else if (rbPennyShape.Checked)
      {
        dummy = Constants.EInclusionKind.PennyShape;
      }
      else if (rbEllipsoid.Checked)
      {
        dummy = Constants.EInclusionKind.Ellipsoid;
      }


      switch (dummy)
      {
        case Constants.EInclusionKind.Sphere:
          dtbA2.Value = dtbA1.Value;
          dtbA3.Value = dtbA1.Value;
          break;
        case Constants.EInclusionKind.Cylinder:
          dtbA2.Value = dtbA1.Value;
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.PennyShape:
          dtbA1.Value = 1;
          dtbA2.Value = 1;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          break;
      }

      _inclusion.SemiAxes[0] = dtbA1.Value;
      _inclusion.SemiAxes[1] = dtbA2.Value;
      _inclusion.SemiAxes[2] = dtbA3.Value;
    }
    private void dtbCi_ValueChanged(object sender, EventArgs e)
    {
      _inclusion.VolFrac = dtbCi.Value;
      CiChanged(this, EventArgs.Empty);
    }
    public void LoadInclusion(Inclusion inclusion)
    {
      _inclusion = inclusion;
      gbInclusionType.Enabled = true;
      dtbCi.Enabled = true;
      tbxName.Enabled = true;
      rbPreferred.Checked = true; 

      dtbA1.Value = _inclusion.SemiAxes[0];
      dtbA2.Value = _inclusion.SemiAxes[1];
      dtbA3.Value = _inclusion.SemiAxes[2];

      dtbFi.Value = _inclusion.EulerAngles[0];
      dtbTheta.Value = _inclusion.EulerAngles[1];
      dtbBeta.Value = _inclusion.EulerAngles[2];

      ucMaterial1.LoadK(_inclusion);
      dtbCi.Value = _inclusion.VolFrac;
      tbxName.Text = _inclusion.name;

      switch (inclusion.InclusionKind)
      {
        case Constants.EInclusionKind.Sphere:
          rbSphere.Checked = true;
          break;
        case Constants.EInclusionKind.Cylinder:
          rbCylinder.Checked = true;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          rbEllipticCylinder.Checked = true;
          break;
        case Constants.EInclusionKind.PennyShape:
          rbPennyShape.Checked = true;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          rbEllipsoid.Checked = true;
          break;
      }
    }

    private void tbxName_TextChanged(object sender, EventArgs e)
    {
      _inclusion.name = tbxName.Text;
      NameChanged(this, EventArgs.Empty);
    }

    public event EventHandler NameChanged;
    public event EventHandler CiChanged;

    private void dtbFi_ValueChanged(object sender, EventArgs e)
    {
      _inclusion.EulerAngles[0] = dtbFi.Value;
      _inclusion.EulerAngles[1] = dtbTheta.Value;
      _inclusion.EulerAngles[2] = dtbBeta.Value;
    }

    public void Clear()
    {
      rbSphere.Checked = false;
      rbCylinder.Checked = false;
      rbEllipticCylinder.Checked = false;
      rbPennyShape.Checked = false;
      rbEllipsoid.Checked = false;
      rbPreferred.Checked = false;
      rbRandom.Checked = false; 

      dtbA1.Clear();
      dtbA2.Clear();
      dtbA3.Clear();

      dtbFi.Clear();
      dtbTheta.Clear();
      dtbBeta.Clear();

      ucMaterial1.Clear();

      tbxName.Clear();
      dtbCi.Clear();
    }

    private void rbPreferredRandom_CheckedChanged(object sender, EventArgs e)
    {
      if (rbPreferred.Checked)
      {
        dtbFi.Enabled = true;
        dtbTheta.Enabled = true;
        dtbBeta.Enabled = true;

        _inclusion.RandomOrientation = false;
      }
      else
      {
        dtbFi.Enabled = false;
        dtbTheta.Enabled = false;
        dtbBeta.Enabled = false;

        _inclusion.RandomOrientation = true;
      }
    }

  }
}
