﻿using System;
using System.Collections.Generic;
using System.Text;

namespace CVUT.Vorel.heat
{
  class Dilute
  {

    public static Tensor Dilute_routine(Matrix A_matrix, Inclusion A_inclusion, double Scale)
    {
      if (Scale <= 0)
      {
        throw new Exception("Heat: Dilute: Dilute_routine: wrong Scale!");
      }

      Tensor K = new Tensor(3, 3);
      Tensor I = new Tensor(3, 3);
      Tensor Tr = new Tensor(3, 3);
      Tensor f = new Tensor(3, 3);
      Tensor T = new Tensor(3, 3); //transformace hl. osy antisymetrie x global;
      Tensor Tt = new Tensor(3, 3); //transformace hl. osy antisymetrie x global;
      Tensor R = new Tensor(3, 3); //transformace inkluze x global 
      Tensor Rt = new Tensor(3, 3); //transformace inkluze x global 
      Tensor L = new Tensor(3, 3); //matrix of ellipsoid L(i,j)x(i)x(j)=1
      Tensor Ki = new Tensor(A_inclusion.HeatConducTensor);
      Tensor ai = new Tensor(A_inclusion.SemiAxes);
      Tensor PrincipalHeatCoef = new Tensor(3);
      double ci = (A_inclusion.VolFrac / Scale);
      long inter;
      int i, j;

      I.Identity();

      // rotations
      A_matrix.HeatConducTensor.Jacobi(ref T, ref PrincipalHeatCoef, 10000, out inter, 1e-20, false);
      Tt = T.Transposition();

      R = A_inclusion.EulerAngles.TransformationEulerAngles_zyz();
      Rt = R.Transposition();

      L = A_inclusion.ShapeTensor;

      L = Tt * Rt * L * R * T;
      Ki = Tt * Rt * Ki * R * T;

      for (i = 0; i < 3; i++)
      {
        for (j = 0; j < 3; j++)
        {
          L[i, j] = L[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
          Ki[i, j] = Ki[i, j] / (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
        }
      }

      Tr = ConcentrationFactorIsotropicUnitMatrix(Ki, L);

      f = Ki - I;   //I is for a scaled matrix heat coef. tensor

      K = I + ci * f * Tr;

      for (i = 0; i < 3; i++)
      {
        for (j = 0; j < 3; j++)
        {
          K[i, j] = K[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
        }
      }
      K = T * K * Tt;

      return (K);
    }
    public static Tensor Dilute_routine(Matrix A_matrix, List<Inclusion> A_list, double Scale)
    {
      Tensor K = new Tensor(3, 3);
      Tensor I = new Tensor(3, 3);
      Tensor Tr = new Tensor(3, 3);
      Tensor f = new Tensor(3, 3);
      Tensor U = new Tensor(3, 3);
      Tensor T = new Tensor(3, 3); //transformace hl. osy antisymetrie x global;
      Tensor Tt = new Tensor(3, 3); //transformace hl. osy antisymetrie x global;
      Tensor R = new Tensor(3, 3); //transformace inkluze x global 
      Tensor Rt = new Tensor(3, 3); //transformace inkluze x global 
      Tensor L = new Tensor(3, 3); //matrix of ellipsoid L(i,j)x(i)x(j)=1
      Tensor Ki = new Tensor(3,3);
      Tensor PrincipalHeatCoef = new Tensor(3);
      double ci,ci_sum = 0;
      long inter;
      int i, j;

      I.Identity();

      // rotations
      A_matrix.HeatConducTensor.Jacobi(ref T, ref PrincipalHeatCoef, 10000, out inter, 1e-20, false);
      Tt = T.Transposition();

      for (int n=0; n < A_list.Count; n++)
      {
        ci = (A_list[n].VolFrac / Scale);
        Ki = A_list[n].HeatConducTensor;

        R = A_list[n].EulerAngles.TransformationEulerAngles_zyz();
        Rt = R.Transposition();

        L = A_list[n].ShapeTensor;

        L = Tt * Rt * L * R * T;
        Ki = Tt * Rt * Ki * R * T;

        for (i = 0; i < 3; i++)
        {

          for (j = 0; j < 3; j++)
          {
            L[i, j] = L[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
            if (!(Math.Abs(Ki[i, j]) < Constants.ZERO))
            {
              Ki[i, j] = Ki[i, j] / (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
            }
          }
        }

        Tr = ConcentrationFactorIsotropicUnitMatrix(Ki, L);

        f = Ki - I;   //I is for a scaled matrix heat coef. tensor

        U += Tensor.ConstantTensorProduct(ci, Tensor.TensorTensorProduct(f, Tr));

        ci_sum += ci;
      }

      if (ci_sum > 1)
      {
        throw new Exception("Heat: Dilute: Dilute_routine: volumu of inclusions is > 1!");
      }

      K = I + U;

      for (i = 0; i < 3; i++)
      {
        for (j = 0; j < 3; j++)
        {
          if (!(K[i, j] == Constants.INFINITY && Math.Abs(PrincipalHeatCoef[i] * PrincipalHeatCoef[j])<Constants.ZERO) &&
              !(Math.Abs(K[i, j]) < Constants.ZERO && (PrincipalHeatCoef[i] * PrincipalHeatCoef[j]) == Constants.INFINITY))
          {
            K[i, j] = K[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
          }
          else
          {
            K[i,j] = 0;
          }
        }
      }
      K = T * K * Tt;

      return (K);
    }

    private static Tensor ConcentrationFactorIsotropicUnitMatrix(Tensor K_inclusion, Tensor L_inclusion)
    {
      Tensor S  = new Tensor(3, 3);
      Tensor Tr = new Tensor(3, 3);
      Tensor H  = new Tensor(3, 3); //transformace hl. osy antisymetrie x hl. osy inkluze
      Tensor Ht = new Tensor(3, 3); //transformace hl. osy antisymetrie x hl. osy inkluze
      Tensor axes = new Tensor(3);
      long inter;

      L_inclusion.Jacobi(ref H, ref axes, 10000, out inter, 1e-20, false);
      Ht = H.Transposition();

      for (int i = 0; i < 3; i++)
      {
        if (Math.Abs(axes[i]) < Constants.ZERO)
          axes[i] = 0;
      }

      axes[0] = Math.Sqrt(1 / axes[0]);
      axes[1] = Math.Sqrt(1 / axes[1]);
      axes[2] = Math.Sqrt(1 / axes[2]);
      if (Double.IsNaN(axes[0]) || Double.IsNaN(axes[1]) || Double.IsNaN(axes[2]))
      {
        throw new Exception("Heat: Dilute: Axis size of inclusion is equal to zero!!!");
      }

      S = Inclusion.GetEshelbyTensor(axes);

      //Tr = (Tensor.GetIdentityTensor(3) + S * (Ht * K_inclusion * H - Tensor.GetIdentityTensor(3))).InverseMatrix();
      Tr = (Tensor.GetIdentityTensor(3) + Tensor.TensorTensorProduct(S, (Tensor.TensorTensorProduct(Tensor.TensorTensorProduct(Ht, K_inclusion), H) - Tensor.GetIdentityTensor(3)))).InverseMatrix();

      Tr = H * Tr * Ht;     // rotace do hl. smeru antisymetrie

      return (Tr);
    }
    private static Tensor OrientationAverage(Tensor A, double Theta)  //It is assume that beta=0 and \phi=0
    {
      if (A.which_order != 2 || A.how_many_lines != 3 || A.how_many_columns != 3)
      {
        throw new Exception("Heat: Dilute: OrientationAverage: wrong tensor!");
      }

      // sin(x) = 2;
      // cos(x) = 3;

      Tensor P1 = new Tensor(3, 3);
      int i, j, k, l;
      double T12;

      double int_ss = Theta - 0.5 * Math.Sin(2 * Theta);
      double int_cc = Theta + 0.5 * Math.Sin(2 * Theta);
      double int_sc = 0.0;
      double int_s = 0.0;
      double int_c = 2 * Math.Sin(Theta);

      int[,] T1 = new int[3, 3];

      //T1[0][0] = 1; T1[0][1] = 0; T1[0][2] = 0; 
      //T1[1][0] = 0; T1[1][1] = 3; T1[1][2] = 2;
      //T1[2][0] = 0; T1[2][1] = -2; T1[2][2] = 3;

      T1[0, 0] = 3; T1[0, 1] = 0; T1[0, 2] = -2;
      T1[1, 0] = 0; T1[1, 1] = 1; T1[1, 2] = 0;
      T1[2, 0] = 2; T1[2, 1] = 0; T1[2, 2] = 3;


      for (i = 0; i < 3; i++)
      {
        for (j = 0; j < 3; j++)
        {
          for (k = 0; k < 3; k++)
          {
            for (l = 0; l < 3; l++)
            {
              switch (T1[k, i] * T1[l, j])
              {
                case 0:
                  T12 = 0.0;
                  break;
                case -1:
                  T12 = -2.0 * Theta;
                  break;
                case -2:
                  T12 = -int_s;
                  break;
                case -3:
                  T12 = -int_c;
                  break;
                case -4:
                  T12 = -int_ss;
                  break;
                case -6:
                  T12 = -int_sc;
                  break;
                case -9:
                  T12 = -int_cc;
                  break;
                case 1:
                  T12 = 2.0 * Theta;
                  break;
                case 2:
                  T12 = int_s;
                  break;
                case 3:
                  T12 = int_c;
                  break;
                case 4:
                  T12 = int_ss;
                  break;
                case 6:
                  T12 = int_sc;
                  break;
                case 9:
                  T12 = int_cc;
                  break;
                default:
                  throw new Exception("Heat: Dilute: OrientationAverage: wrong case!");
              }
              P1[i, j] += 1 / (2 * Theta) * T12 * A[k, l];
            }
          }
        }
      }
      return (P1);
    }
    private static Tensor OrientationAverage(Tensor A, DataXY histogram)
    {
      if (A.which_order != 2 || A.how_many_lines != 3 || A.how_many_columns != 3)
      {
        throw new Exception("Heat: Dilute: OrientationAverage: wrong tensor!");
      }

      Tensor P1 = new Tensor(3, 3);
      Tensor T1 = new Tensor(3, 3);

      for (int i = 0; i < histogram.Count; i++)
      {
        T1[0, 0] = Math.Cos(histogram.x[i]); T1[0, 1] = 0; T1[0, 2] = -Math.Sin(histogram.x[i]);
        T1[1, 0] = 0;                        T1[1, 1] = 1; T1[1, 2] = 0;
        T1[2, 0] = Math.Sin(histogram.x[i]); T1[2, 1] = 0; T1[2, 2] = Math.Cos(histogram.x[i]);

        P1 = P1 + histogram.y[i] * T1.Transposition() * A * T1;
      }

      return (P1);
    }

  }
}

