﻿using System;
using System.Collections.Generic;
using System.Text;

namespace CVUT.Vorel.heat
{
  class SC
  {

    public static Tensor SC_routine(Matrix A_matrix, Inclusion A_inclusion, double Scale)
    {
      if (Scale <= 0)
      {
        throw new Exception("Heat: SC: SC_routine: wrong Scale!");
      }

      Tensor K = new Tensor(3, 3);
      Tensor I = new Tensor(3, 3);
      Tensor Tr = new Tensor(3, 3);
      Tensor f = new Tensor(3, 3);
      Tensor T = new Tensor(3, 3); //transformace hl. osy antisymetrie x global;
      Tensor Tt = new Tensor(3, 3); //transformace hl. osy antisymetrie x global;
      Tensor R = new Tensor(3, 3); //transformace inkluze x global 
      Tensor Rt = new Tensor(3, 3); //transformace inkluze x global 
      Tensor L = new Tensor(3, 3); //matrix of ellipsoid L(i,j)x(i)x(j)=1
      Tensor K0 = new Tensor(A_matrix.HeatConducTensor);
      Tensor K0dum = new Tensor(A_matrix.HeatConducTensor);
      Tensor Ki = new Tensor(A_inclusion.HeatConducTensor);
      Tensor Ki0 = new Tensor(A_inclusion.HeatConducTensor);
      Tensor ai = new Tensor(A_inclusion.SemiAxes);
      Tensor PrincipalHeatCoef = new Tensor(3);
      Matrix dum_matrix = new Matrix(A_matrix);
      double ci = (A_inclusion.VolFrac / Scale);
      long inter;
      int i, j;

      double err = 1.0, tol0 = 1.0e-6, tol;
      int maxIter = 10000, nIter = 0;

      I.Identity();

      R = A_inclusion.EulerAngles.TransformationEulerAngles_zyz();
      Rt = R.Transposition();

      tol   = tol0 * Math.Min(Tensor.MaxAbsElement(dum_matrix.HeatConducTensor), Tensor.MaxAbsElement(Ki));
      err   = Tensor.MaxAbsElement(dum_matrix.HeatConducTensor);
      K     = A_matrix.HeatConducTensor;
      nIter = 0;
      try
      {
        while ((err > tol && nIter < maxIter) || nIter == 0)
        {
          nIter++;

          // rotations
          K.Jacobi(ref T, ref PrincipalHeatCoef, 10000, out inter, 1e-20, false);
          Tt = T.Transposition();

          L = A_inclusion.ShapeTensor;
          L = Tt * Rt * L * R * T;
          Ki = Tt * Rt * Ki0 * R * T;

          for (i = 0; i < 3; i++)
          {
            for (j = 0; j < 3; j++)
            {
              L[i, j] = L[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
              Ki[i, j] = Ki[i, j] / (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
              K0dum[i, j] = K0[i, j] / (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
            }
          }

          Tr = ConcentrationFactorIsotropicUnitMatrix(Ki, L);

          f = Ki - K0dum; 

          K = K0dum + ci * f * Tr;

          for (i = 0; i < 3; i++)
          {
            for (j = 0; j < 3; j++)
            {
              K[i, j] = K[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
            }
          }
          K = T * K * Tt;

          if (nIter == 1)
          {
            tol = Math.Min(tol, tol0 * Tensor.MaxElement(K));
          }
          err = Tensor.MaxAbsElement(K - dum_matrix.HeatConducTensor);
          dum_matrix.HeatConducTensor = K;
        }
        if (nIter >= maxIter)
        {
          throw new Exception("Heat: SC: SC_routine: Maximum number (" + Convert.ToString(maxIter, Constants.Separators.FormatProvider) +
                              " of iterations was exceeded!");
        }
      }
      catch
      {
        // swaping phases + rotation adjustment 
        K0 = Rt * A_inclusion.HeatConducTensor * R;
        Ki0 = R * A_matrix.HeatConducTensor * Rt; // rotate to local CS

        dum_matrix = new Matrix(A_matrix);
        dum_matrix.HeatConducTensor = K0;

        ci = 1 - ci;

        err = Tensor.MaxAbsElement(dum_matrix.HeatConducTensor);
        K = K0;
        nIter = 0;
        while ((err > tol && nIter < maxIter) || nIter == 0)
        {
          nIter++;

          // rotations
          K.Jacobi(ref T, ref PrincipalHeatCoef, 10000, out inter, 1e-20, false);
          Tt = T.Transposition();

          L = A_inclusion.ShapeTensor;
          L = Tt * Rt * L * R * T;
          Ki = Tt * Rt * Ki0 * R * T;

          for (i = 0; i < 3; i++)
          {
            for (j = 0; j < 3; j++)
            {
              L[i, j] = L[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
              Ki[i, j] = Ki[i, j] / (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
              K0dum[i, j] = K0[i, j] / (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
            }
          }

          Tr = ConcentrationFactorIsotropicUnitMatrix(Ki, L);

          f = Ki - K0dum;   //I is for a scaled matrix heat coef. tensor

          K = K0dum + ci * f * Tr;

          for (i = 0; i < 3; i++)
          {
            for (j = 0; j < 3; j++)
            {
              K[i, j] = K[i, j] * (Math.Sqrt(PrincipalHeatCoef[i] * PrincipalHeatCoef[j]));
            }
          }
          K = T * K * Tt;

          err = Tensor.MaxAbsElement(K - dum_matrix.HeatConducTensor);
          dum_matrix.HeatConducTensor = K;
        }
        if (nIter >= maxIter)
        {
          throw new Exception("Heat: SC: SC_routine: Maximum number (" + Convert.ToString(maxIter, Constants.Separators.FormatProvider) +
                              " of iterations was exceeded!");
        }
      }

      return (K);
    }
    public static Tensor SC_routine(Matrix A_matrix, List<Inclusion> A_list, double Scale)
    {
      // Only one inclusion is allowed 
      Tensor K = new Tensor(3, 3);

      if (A_list.Count > 0)
      {
        K = SC_routine(A_matrix, A_list[0], Scale);
      }
      else
      {
        K = A_matrix.HeatConducTensor;
      }

      return (K);
    }

    private static Tensor ConcentrationFactorIsotropicUnitMatrix(Tensor K_inclusion, Tensor L_inclusion)
    {
      Tensor S  = new Tensor(3, 3);
      Tensor Tr = new Tensor(3, 3);
      Tensor H  = new Tensor(3, 3); //transformace hl. osy antisymetrie x hl. osy inkluze
      Tensor Ht = new Tensor(3, 3); //transformace hl. osy antisymetrie x hl. osy inkluze
      Tensor axes = new Tensor(3);
      long inter;

      L_inclusion.Jacobi(ref H, ref axes, 10000, out inter, 1e-20, false);
      Ht = H.Transposition();

      for (int i = 0; i < 3; i++)
      {
        if (Math.Abs(axes[i]) < Constants.ZERO)
          axes[i] = 0;
      }

      axes[0] = Math.Sqrt(1 / axes[0]);
      axes[1] = Math.Sqrt(1 / axes[1]);
      axes[2] = Math.Sqrt(1 / axes[2]);
      if (Double.IsNaN(axes[0]) || Double.IsNaN(axes[1]) || Double.IsNaN(axes[2]))
      {
        throw new Exception("Heat: SC: Axis size of inclusion is equal to zero!!!");
      }

      S = Inclusion.GetEshelbyTensor(axes);

      //Tr = (Tensor.GetIdentityTensor(3) + S * (Ht * K_inclusion * H - Tensor.GetIdentityTensor(3))).InverseMatrix();
      Tr = (Tensor.GetIdentityTensor(3) + Tensor.TensorTensorProduct(S, (Tensor.TensorTensorProduct(Tensor.TensorTensorProduct(Ht, K_inclusion), H) - Tensor.GetIdentityTensor(3)))).InverseMatrix();

      Tr = H * Tr * Ht;     // rotace do hl. smeru antisymetrie

      return (Tr);
    }
    private static Tensor OrientationAverage(Tensor A, double Theta)  //It is assume that beta=0 and \phi=0
    {
      if (A.which_order != 2 || A.how_many_lines != 3 || A.how_many_columns != 3)
      {
        throw new Exception("Heat: SC: OrientationAverage: wrong tensor!");
      }

      // sin(x) = 2;
      // cos(x) = 3;

      Tensor P1 = new Tensor(3, 3);
      int i, j, k, l;
      double T12;

      double int_ss = Theta - 0.5 * Math.Sin(2 * Theta);
      double int_cc = Theta + 0.5 * Math.Sin(2 * Theta);
      double int_sc = 0.0;
      double int_s = 0.0;
      double int_c = 2 * Math.Sin(Theta);

      int[,] T1 = new int[3, 3];

      //T1[0][0] = 1; T1[0][1] = 0; T1[0][2] = 0; 
      //T1[1][0] = 0; T1[1][1] = 3; T1[1][2] = 2;
      //T1[2][0] = 0; T1[2][1] = -2; T1[2][2] = 3;

      T1[0, 0] = 3; T1[0, 1] = 0; T1[0, 2] = -2;
      T1[1, 0] = 0; T1[1, 1] = 1; T1[1, 2] = 0;
      T1[2, 0] = 2; T1[2, 1] = 0; T1[2, 2] = 3;


      for (i = 0; i < 3; i++)
      {
        for (j = 0; j < 3; j++)
        {
          for (k = 0; k < 3; k++)
          {
            for (l = 0; l < 3; l++)
            {
              switch (T1[k, i] * T1[l, j])
              {
                case 0:
                  T12 = 0.0;
                  break;
                case -1:
                  T12 = -2.0 * Theta;
                  break;
                case -2:
                  T12 = -int_s;
                  break;
                case -3:
                  T12 = -int_c;
                  break;
                case -4:
                  T12 = -int_ss;
                  break;
                case -6:
                  T12 = -int_sc;
                  break;
                case -9:
                  T12 = -int_cc;
                  break;
                case 1:
                  T12 = 2.0 * Theta;
                  break;
                case 2:
                  T12 = int_s;
                  break;
                case 3:
                  T12 = int_c;
                  break;
                case 4:
                  T12 = int_ss;
                  break;
                case 6:
                  T12 = int_sc;
                  break;
                case 9:
                  T12 = int_cc;
                  break;
                default:
                  throw new Exception("Heat: SC: OrientationAverage: wrong case!");
              }
              P1[i, j] += 1 / (2 * Theta) * T12 * A[k, l];
            }
          }
        }
      }
      return (P1);
    }
    private static Tensor OrientationAverage(Tensor A, DataXY histogram)
    {
      if (A.which_order != 2 || A.how_many_lines != 3 || A.how_many_columns != 3)
      {
        throw new Exception("Heat: SC: OrientationAverage: wrong tensor!");
      }

      Tensor P1 = new Tensor(3, 3);
      Tensor T1 = new Tensor(3, 3);

      for (int i = 0; i < histogram.Count; i++)
      {
        T1[0, 0] = Math.Cos(histogram.x[i]); T1[0, 1] = 0; T1[0, 2] = -Math.Sin(histogram.x[i]);
        T1[1, 0] = 0;                        T1[1, 1] = 1; T1[1, 2] = 0;
        T1[2, 0] = Math.Sin(histogram.x[i]); T1[2, 1] = 0; T1[2, 2] = Math.Cos(histogram.x[i]);

        P1 = P1 + histogram.y[i] * T1.Transposition() * A * T1;
      }

      return (P1);
    }

  }
}

