﻿using System;
using System.Collections.Generic;
using System.Text;
using CVUT.Vorel;

namespace CVUT.Vorel.heat
{

  public class Constituent
  {
    private Tensor _HeatConductTensor;

    /////////////////////////////////////////////////////////////////////////////////////////////
    public Constituent()
    {
      _HeatConductTensor = new Tensor(3, 3);
    }
    public Constituent(double HeatCoef)
    {
      _HeatConductTensor = new Tensor(3, 3);
      _HeatConductTensor[0, 0] = _HeatConductTensor[1, 1] = _HeatConductTensor[2, 2] = HeatCoef;
    }
    public Constituent(double HeatCoef0, double HeatCoef1, double HeatCoef2)
    {
      _HeatConductTensor = new Tensor(3, 3);
      _HeatConductTensor[0, 0] = HeatCoef0;
      _HeatConductTensor[1, 1] = HeatCoef1;
      _HeatConductTensor[2, 2] = HeatCoef2;
    }
    public Constituent(Tensor HeatCoef)
    {
      if (HeatCoef.which_order!= 2 || HeatCoef.how_many_lines!= 3 || HeatCoef.how_many_columns!= 3)
      {
        throw new Exception("Heat: Constituent: Constituent(Tensor HeatCoef): wrong tensor!");
      }

      _HeatConductTensor = new Tensor(HeatCoef);
    }
    public Constituent(Constituent oa) 
      : this(oa._HeatConductTensor)
    {
    }
    /////////////////////////////////////////////////////////////////////////////////////////////
    public Tensor HeatConducTensor
    {
      get
      {
        return (_HeatConductTensor);
      }
      set
      {
        _HeatConductTensor = value;
      }
    }
    public Constants.EMaterialKind MaterialKind
    {
      get
      {

        if (Math.Abs(_HeatConductTensor[0, 1]) < Constants.ZERO && Math.Abs(_HeatConductTensor[0, 2]) < Constants.ZERO &&
            Math.Abs(_HeatConductTensor[1, 0]) < Constants.ZERO && Math.Abs(_HeatConductTensor[1, 2]) < Constants.ZERO &&
            Math.Abs(_HeatConductTensor[2, 0]) < Constants.ZERO && Math.Abs(_HeatConductTensor[2, 1]) < Constants.ZERO)
        {
          if (Math.Abs(_HeatConductTensor[0, 0] - _HeatConductTensor[1, 1]) < Constants.ZERO &&
              Math.Abs(_HeatConductTensor[0, 0] - _HeatConductTensor[2, 2]) < Constants.ZERO)
          {
            return Constants.EMaterialKind.Isotropic;
          }
          else
          {
            return Constants.EMaterialKind.Orthotropic;
          }
        }
        else
        {
          if (Math.Abs(_HeatConductTensor[0, 1] - Math.Abs(_HeatConductTensor[1, 0])) < Constants.ZERO &&
              Math.Abs(_HeatConductTensor[0, 2] - Math.Abs(_HeatConductTensor[2, 0])) < Constants.ZERO &&
              Math.Abs(_HeatConductTensor[1, 2] - Math.Abs(_HeatConductTensor[2, 1])) < Constants.ZERO)
          {
            return Constants.EMaterialKind.Anisotropic;
          }
          else
          {
            return Constants.EMaterialKind.FullyAnisotropic;
          }
        }
      }
    }
    public void ClearConstituent()
    {
      this._HeatConductTensor.Clear();
    }

  }
}
