﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;

namespace CVUT.Vorel.heat
{

  public class Inclusion : Constituent
  {
    private Tensor _a;																							// semi-axes
    private double _VolumeFrac;																			// volume fracture of the whole structure
    private Tensor _EulerAngles;																		// Euler angles-phi, theta, beta
    private bool _RandomOrientation;			     										  // defines if the random orientation will be used
    public string name = "";

    /////////////////////////////////////////////////////////////////////////////////////////////
    public Inclusion()
      : base()
    {
      _a = new Tensor(3);
      _a[0]=_a[1]=_a[2] = 1;
      _EulerAngles = new Tensor(3);

      _VolumeFrac = 0.0;
      _RandomOrientation = false;
    }
    public Inclusion(double a0, double a1, double a2)
      : base()
    {
      _a = new Tensor(3);
      _EulerAngles = new Tensor(3);

      _a[0] = a0;
      _a[1] = a1;
      _a[2] = a2;

      _VolumeFrac = 0.0;
      _RandomOrientation = false;
    }
    public Inclusion(double a0, double a1, double a2, double HeatCoef, double VolumeFr)
      : base(HeatCoef)
    {
      _a = new Tensor(3);
      _EulerAngles = new Tensor(3);

      _a[0] = a0;
      _a[1] = a1;
      _a[2] = a2;

      _VolumeFrac = VolumeFr;
      _RandomOrientation = false;
    }
    public Inclusion(double a0, double a1, double a2, Tensor HeatCoef, double VolumeFr)
      : base(HeatCoef)
    {
      _a = new Tensor(3);
      _EulerAngles = new Tensor(3);

      _a[0] = a0;
      _a[1] = a1;
      _a[2] = a2;

      _VolumeFrac = VolumeFr;
      _RandomOrientation = false;
    }
    public Inclusion(double a0, double a1, double a2, double HeatCoef0, double HeatCoef1, double HeatCoef2, double VolumeFr)
      : base(HeatCoef0, HeatCoef1, HeatCoef2)
    {
      _a = new Tensor(3);
      _EulerAngles = new Tensor(3);

      _a[0] = a0;
      _a[1] = a1;
      _a[2] = a2;

      _VolumeFrac = VolumeFr;
      _RandomOrientation = false;
    }
    public Inclusion(Inclusion oa)
      : base(oa)
    {
      _a = new Tensor(oa._a);
      _EulerAngles = new Tensor(oa._EulerAngles);

      _VolumeFrac = oa._VolumeFrac;
      _RandomOrientation = oa._RandomOrientation;
    }
    public Inclusion(Tensor a, Tensor HeatCoef, double VolumeFr, Tensor EulerAngles)
      : base(HeatCoef)
    {
      _a = new Tensor(a);
      _EulerAngles = new Tensor(EulerAngles);

      _VolumeFrac = VolumeFr;
      _RandomOrientation = false;
    }
    /////////////////////////////////////////////////////////////////////////////////////////////
    public Tensor SemiAxes
    {
      get
      {
        return (_a);
      }
    }
    public Tensor EulerAngles
    {
      get
      {
        return (_EulerAngles);
      }
    }
    public double VolFrac
    {
      get
      {
        return (_VolumeFrac);
      }
      set
      {
        _VolumeFrac = value;
      }
    }
    public bool RandomOrientation
    {
      get
      {
        return (_RandomOrientation);
      }
      set
      {
        _RandomOrientation = value;
      }
    }
    public Tensor ShapeTensor
    {
      get
      {
        Tensor L = new Tensor(3, 3);

        L[0, 0] = 1 / (_a[0] * _a[0]);
        L[1, 1] = 1 / (_a[1] * _a[1]);
        L[2, 2] = 1 / (_a[2] * _a[2]);

        return (L);
      }
    }
    public Constants.EInclusionKind InclusionKind
    {
      get
      {
        return this.GetShapeKind();
      }
    }
    private Constants.EInclusionKind GetShapeKind()
    {
      Tensor a = new Tensor(this._a);
      int seq;

      a = a.Sort(out seq);

      if (Math.Abs(a[0] - a[1]) < Constants.ZERO && Math.Abs(a[0] - a[2]) < Constants.ZERO)
      {
        return (Constants.EInclusionKind.Sphere);
      }
      else if (a[0] == Constants.INFINITY)
      {
        if (Math.Abs(a[1] - a[2]) < Constants.ZERO)
        {
          return (Constants.EInclusionKind.Cylinder);
        }
        else
        {
          return (Constants.EInclusionKind.EllipticCylinder);
        }
      }
      else if (Math.Abs(a[0] - a[1]) < Constants.ZERO && Functions.IsMuchMore(a[0], a[2]))
      {
        return (Constants.EInclusionKind.PennyShape);
      }
      else
      {
        return (Constants.EInclusionKind.Ellipsoid);
      }
    }
    /////////////////////////////////////////////////////////////////////////////////////////////
    private void ChangeSequenceAsGiven(Tensor A, int A_sequence)
    {
      if (A.which_order != 2 || A.how_many_lines != 3 || A.how_many_columns != 3)
      {
        throw new Exception("Inclusion: ChangeSequenceAsGiven(Tensor A, int A_sequence): wrong tensor!");
      }

      double dummy;

      switch (A_sequence)
      {
        case 0:
          break;

        case 1:
          dummy = A[1, 1];
          A[1, 1] = A[2, 2];
          A[2, 2] = dummy;
          break;

        case 2:
          dummy = A[0, 0];
          A[0, 0] = A[1, 1];
          A[1, 1] = dummy;
          break;

        case 3:
          dummy = A[0, 0];
          A[0, 0] = A[2, 2];
          A[2, 2] = A[1, 1];
          A[1, 1] = dummy;
          break;

        case 4:
          dummy = A[0, 0];
          A[0, 0] = A[1, 1];
          A[1, 1] = A[2, 2];
          A[2, 2] = dummy;
          break;

        case 5:
          dummy = A[0, 0];
          A[0, 0] = A[2, 2];
          A[2, 2] = dummy;
          break;

        default:
          throw new Exception("Inclusion: ChangeSequenceAsGiven(): wrong case!");
      }
    }
    public static Tensor GetEshelbyTensor(Tensor a)  //for inclusion in isotropic matrix
    {
      if (a.which_order != 1 || a.how_many_lines != 3)
      {
        throw new Exception("Heat: Inclusion: GetEshelbyTensor: wrong tensor!");
      }

      Tensor S = new Tensor(3,3);
      int sequence;

      a = a.Sort(out sequence);				// a0>a1>a3

      if (a[0] == a[1] && a[0] == a[2])
      {
        S[0,0] = S[1,1] = S[2,2] = 1.0/3.0;
      }
      else if (a[0] == Constants.INFINITY)
      {
        if (a[1] == Constants.INFINITY)
        {
          S[0, 0] = S[1, 1] = 0.0;
          S[2, 2] = 1.0;
        }
        else
        {
          S[1, 1] = a[2] / (a[1] + a[2]);
          S[2, 2] = a[1] / (a[1] + a[2]);
          S[0, 0] = 0.0;
        }
      }
      else if (a[0] == a[1] && Functions.IsMuchMore(a[0], a[2]))
      {
        S[0,0] = S[1, 1] = Math.PI * a[2] / (4.0 * a[0]);
        S[2,2] = 1.0 - Math.PI * a[2] / (2.0 * a[1]);
      }
      else if (a[0] == a[1] && a[0] > a[2])
      {
        S[0,0] = S[1,1] = a[0] * a[0] * a[2] / (2.0 * Math.Pow(a[0] * a[0] - a[2] * a[2], 1.5))
                                                    * (Math.Acos(a[2] / a[0]) - a[2] / a[0] * Math.Pow(1.0 - a[2] * a[2] / (a[0] * a[0]), 0.5));
        S[2,2] = 1.0 - 2.0 * S[0, 0];
      }
      else if (a[1] == a[2])
      {
        S[1,1] = S[2,2] = a[1] * a[1] * a[0] / (2.0 * Math.Pow(a[0] * a[0] - a[1] * a[1], 1.5))
                                                    * (a[0] / a[1] * Math.Pow(a[0] * a[0] / (a[1] * a[1]) - 1.0, 0.5) - Functions.ACosh(a[0] / a[1]));
        S[0,0] = 1.0 - 2.0 * S[1,1];
      }
      else
      {
        double E_int, F_int, theta, k;
        double a02, a12, a22;

        a02 = a[0] * a[0];
        a12 = a[1] * a[1];
        a22 = a[2] * a[2];

        k = ((a02 - a12) / (a02 - a22));
        theta = Math.Asin(Math.Sqrt(1.0 - a22 / a02));

        F_int = Elliptic.incompleteellipticintegralk(theta, k);
        E_int = Elliptic.incompleteellipticintegrale(theta, k);

        S[0,0] = a[0] * a[1] * a[2] / ((a02 - a12) * Math.Sqrt(a02 - a22)) * (F_int - E_int);
        S[1,1] = a[0] * a[1] * a[2] * (Math.Sqrt(a02 - a22) / ((a02 - a12) * (a12 - a22)) * E_int
                              - F_int / ((a02 - a12) * Math.Sqrt(a02 - a22))
                              - a[2] / (a[0] * a[1] * (a12 - a22)));
        S[2,2] = a[0] * a[1] * a[2] * (E_int / ((a22 - a12) * Math.Sqrt(a02 - a22))
                              + a[1] / (a[0] * a[2] * (a12 - a22)));
        //S[1,1] = 1-S[0,0]-S[2,2];
      }

      // origin sequence

      S = S.Unsort(sequence);
      return (S);
    }
    public override string ToString()
    {
      return name;
    }
    public void ClearInclusion()
    {
      base.ClearConstituent();
      _a[0] = _a[1] = _a[2] = 1;
      this._EulerAngles.Clear();
      this._VolumeFrac = 0;
      this.name = "";
      this._RandomOrientation = false;
    }

    public XmlElement ToXml(XmlDocument xmldoc)
    {
      XmlElement _incl;
      XmlElement _text1;
      XmlAttribute _atrib;

      _incl = xmldoc.CreateElement("inclusion");
      
      _atrib = xmldoc.CreateAttribute("name");
      _atrib.Value = name;
      _incl.SetAttributeNode(_atrib);

      _text1 = xmldoc.CreateElement("vol_frac");
      _text1.InnerText = Convert.ToString(VolFrac, Constants.Separators.FormatProvider);
      _incl.AppendChild(_text1);

      _incl.AppendChild(SemiAxes.ToXml(xmldoc, "semi_axes"));

      _incl.AppendChild(EulerAngles.ToXml(xmldoc, "Euler_angles"));

      _incl.AppendChild(HeatConducTensor.ToXml(xmldoc, "heat_conduction"));

      return (_incl);
    }
    public void FromXml(XmlNode xmlnode)
    {
      string name = "";
      this.ClearInclusion();
      int[] count = new int[4];

      count[0] = count[1] = count[2] = count[3] = 0;

      this.name = "new inclusion";
      for (int i = 0; i < xmlnode.Attributes.Count; i++)
      {
        if (xmlnode.Attributes[i].Name == "name")
        {
          this.name = xmlnode.Attributes[i].Value;
        }
      }

      if (!(xmlnode.ChildNodes.Count == 4))
      {
        throw new Exception("xml: inclusion: wrong number of children!!!");
      }
      else
      {
        for (int i = 0; i < 4; i++)
        {
          switch (xmlnode.ChildNodes[i].Name)
          {
            case "vol_frac":
              count[0]++;
              _VolumeFrac = Functions.StringToDouble(xmlnode.ChildNodes[i].InnerText, "xml: inclusion: wrong vol_frac!!!");
              break;

            case "tensor":
              for (int j = 0; j < xmlnode.ChildNodes[i].Attributes.Count; j++)
              {
                if (xmlnode.ChildNodes[i].Attributes[j].Name == "name")
                {
                  name = xmlnode.ChildNodes[i].Attributes[j].Value;
                }
              }
              switch (name)
              {
                case "semi_axes":
                  count[1]++;
                  this.SemiAxes.FromXml(xmlnode.ChildNodes[i]);
                  break;

                case "Euler_angles":
                  count[2]++;
                  this.EulerAngles.FromXml(xmlnode.ChildNodes[i]);
                  break;

                case "heat_conduction":
                  count[3]++;
                  this.HeatConducTensor.FromXml(xmlnode.ChildNodes[i]);
                  break;

                default:
                  throw new Exception("xml: inclusion: wrong tensor name!!!");
              }
              break;

            default:
              throw new Exception("xml: inclusion: wrong tensor child!!!");
          }

        }
        if (!(count[0] == 1 && count[1] == 1 && count[2] == 1 && count[3] == 1))
        {
          throw new Exception("xml: inclusion: there miss some inclusion parameters!!!");
        }

      }
    }

    //private Tensor GetEshelbyTensor(Tensor MatrixHeatCoef)			//MatrixHeatCoef is a diagonal matrix 
    //{
    //  if (MatrixHeatCoef[0, 1] != 0 || MatrixHeatCoef[0, 2] != 0 || MatrixHeatCoef[1, 0] != 0 ||
    //      MatrixHeatCoef[1, 2] != 0 || MatrixHeatCoef[2, 0] != 0 || MatrixHeatCoef[2, 1] != 0)
    //  {
    //    throw new Exception("Inclusion: GetEshelbyTensor(Tensor MatrixHeatCoef): Isn't orthogonal matrix!");
    //  }

    //  Tensor Eshelby = new Tensor(3, 3);
    //  Tensor a = new Tensor(3);
    //  int a_sequence = 0;
    //  int i;

    //  for (i = 0; i < 3; i++)
    //  {
    //    if (_a[i] > 0)
    //    {
    //      if (MatrixHeatCoef[i, i] == 0)
    //      {
    //        throw new Exception("Inclusion: GetEshelbyTensor(Tensor MatrixHeatCoef): MatrixHeatCoef==0!");
    //      }
    //      else
    //      {
    //        a[i] = _a[i] / Math.Sqrt(MatrixHeatCoef[i, i]);
    //      }
    //    }
    //    else
    //    {
    //      a[i] = _a[i];				// for infinity
    //    }
    //  }

    //  Sort(ref a, ref a_sequence);
    //  ChangeSequenceAsGiven(MatrixHeatCoef, a_sequence);

    //  if (a[0] == a[1] && a[0] == a[2])
    //  {
    //    Eshelby[0, 0] = Eshelby[1, 1] = Eshelby[2, 2] = 1 / 3;
    //  }
    //  else if (a[2] == INFINITY)
    //  {
    //    Eshelby[0, 0] = a[1] / (a[0] + a[1]);
    //    Eshelby[1, 1] = a[0] / (a[0] + a[1]);
    //    Eshelby[2, 2] = 0.0;
    //  }
    //  else if (a[0] == a[1] && IsMuchMore(a[0], a[2]))
    //  {
    //    Eshelby[0, 0] = Eshelby[1, 1] = Math.PI * a[2] / (4 * a[0]);
    //    Eshelby[2, 2] = 1 - Math.PI * a[2] / (2 * a[1]);
    //  }
    //  else if (a[0] == a[1] && a[0] > a[2])
    //  {
    //    Eshelby[0, 0] = Eshelby[1, 1] = a[0] * a[0] * a[2] / (2 * Math.Pow(a[0] * a[0] - a[2] * a[2], 1.5))
    //                                                * (Math.Acos(a[2] / a[0]) - a[2] / a[0] * Math.Pow(1 - a[2] * a[2] / (a[0] * a[0]), 0.5));
    //    Eshelby[2, 2] = 1 - 2 * Eshelby[0, 0];
    //  }
    //  else if (a[1] == a[2])
    //  {
    //    Eshelby[1, 1] = Eshelby[2, 2] = a[1] * a[1] * a[0] / (2 * Math.Pow(a[0] * a[0] - a[1] * a[1], 1.5))
    //                                                * (a[0] / a[1] * Math.Pow(a[0] * a[0] / (a[1] * a[1]) - 1, 0.5) - ACosh(a[0] / a[1]));
    //    Eshelby[0, 0] = 1 - 2 * Eshelby[1, 1];
    //  }
    //  else
    //  {
    //    double E_int, F_int, theta, k;
    //    double a02, a12, a22;
    //    double k012;

    //    a02 = a[0] * a[0];
    //    a12 = a[1] * a[1];
    //    a22 = a[2] * a[2];

    //    k = ((a02 - a12) / (a02 - a22));
    //    theta = Math.Asin(Math.Sqrt(1 - a22 / a02));

    //    F_int = Elliptic.incompleteellipticintegralk(theta, k);
    //    E_int = Elliptic.incompleteellipticintegrale(theta, k);

    //    k012 = Math.Sqrt(MatrixHeatCoef[0, 0] * MatrixHeatCoef[1, 1] * MatrixHeatCoef[2, 2]);

    //    Eshelby[0, 0] = a[0] * a[1] * a[2] / ((a02 - a12) * Math.Sqrt(a02 - a22)) * (F_int - E_int);
    //    Eshelby[1, 1] = a[0] * a[1] * a[2] * (
    //                   Math.Sqrt(a02 - a22) / ((a02 - a12) * (a12 - a22)) * E_int
    //                   - F_int / ((a02 - a12) * Math.Sqrt(a02 - a22))
    //                   - a[2] / (a[0] * a[1] * (a12 - a22)));
    //    Eshelby[2, 2] = a[0] * a[1] * a[2] * (
    //                   E_int / ((a22 - a12) * Math.Sqrt(a02 - a22))
    //                   + a[1] / (a[0] * a[2] * (a12 - a22)));
    //    //Eshelby[1,1] = 1-Eshelby[0,0]-Eshelby[2,2];
    //  }

    //  RestoreSequence(ref Eshelby, ref a_sequence);

    //  return Eshelby;
    //}
  }
}
