﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CVUT.Vorel
{
  public static class Common
  {
    public static double Min(List<double> list)
    {
      double min = Double.PositiveInfinity;
      foreach (double current in list)
        if (current < min)
          min = current;
      return min;
    }
    public static double Max(List<double> list)
    {
      double max = Double.NegativeInfinity;
      foreach (double current in list)
        if (current > max)
          max = current;
      return max;
    }
    public static string DoubleToString(double value)
    {
      if (value == 0.0)
        return value.ToString("0");
      if (Math.Abs(value) >= 10000)
        return value.ToString("0.##e+0");
      if (Math.Abs(value) >= 1000)
        return Math.Round(value, 0).ToString();
      if (Math.Abs(value) >= 100)
        return Math.Round(value, 1).ToString();
      if (Math.Abs(value) >= 10)
        return Math.Round(value, 2).ToString();
      if (Math.Abs(value) >= 1.0)
        return Math.Round(value, 3).ToString();
      if (Math.Abs(value) >= .1)
        return Math.Round(value, 4).ToString();

      return value.ToString("0.##e+0");
    }
    public static double RoundToDigits(double value, int digits)
    {
      return Math.Round(value / Math.Pow(10.0, Math.Floor(Math.Log10(Math.Abs(value)))), digits - 1) * Math.Pow(10.0, Math.Floor(Math.Log10(Math.Abs(value))));
    }
    public static string[] SplittingLine(string line, int nItems)
    {
      string[] items_in_line = new string[nItems];

      // Splitting line
      items_in_line = line.Split(new Char[] { ';' });
      if (items_in_line.GetLength(0) < nItems)
      {
        items_in_line = line.Split(new Char[] { '\t' });

        if (items_in_line.GetLength(0) < nItems)
        {
          items_in_line = line.Split(new Char[] { ' ' });

          if (items_in_line.GetLength(0) < nItems)
          {
            throw new Exception("SplittingLine: There is only one number in line or wrong separator!!!");
          }
        }
      }

      return (items_in_line);
    }
    public static Tensor StringArrayToTensor(string[] items_in_line, int nItems)
    {
      Tensor a = new Tensor(nItems);

      for (int i = 0; i < nItems; i++)
      {
        if (items_in_line[i].ToLower() == "infinity")
        {
          items_in_line[i] = Convert.ToString(Double.PositiveInfinity, Constants.Separators.FormatProvider);
        }
        else if (items_in_line[i].ToLower() == "-infinity")
        {
          items_in_line[i] = Convert.ToString(Double.NegativeInfinity, Constants.Separators.FormatProvider);
        }
        a[i] = Functions.StringToDouble(items_in_line[i], "StringArrayToTensor: It is not a number!!!");
      }

      return (a);
    }
    public static Constants.EProblemKind GetProblemKind(string name)
    {
      switch (name.ToLower())
      {
        case "heat":
          return Constants.EProblemKind.Heat;
        case "elasticity":
          return Constants.EProblemKind.Elasticity;
        default:
          return Constants.EProblemKind.Empty;
      }
    }
    public static int GetNumberOfIndependentForElasticity(Constants.EMaterialKind MatKind)
    {
      switch (MatKind)
      {
        case Constants.EMaterialKind.Isotropic:
          return (2);
        case Constants.EMaterialKind.Orthotropic:
          return (9);
        case Constants.EMaterialKind.Anisotropic:
          return (21);
        case Constants.EMaterialKind.FullyAnisotropic:
          return (36);
        default:
          throw new Exception("Constants: GetNumberOfIndependentForElasticity: wrong material!!!");
      }
    }  

  }
}
