﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CVUT.Vorel
{
  public interface ILog
  {
    void Write(string text);
    void WriteLine(string line);
    void WriteLine();
  }

  public class LogLabel : System.Windows.Forms.TextBox, ILog
  {
    public LogLabel()
      : base()
    {
      ReadOnly = true;
      BackColor = System.Drawing.Color.White;
      Text = "";
      Multiline = true;
      WordWrap = false;
      BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
      ScrollBars = System.Windows.Forms.ScrollBars.Both;
    }
    public void Write(string text)
    {
      AppendText(text);
      SelectionStart = Text.Length;
      ScrollToCaret();
    }
    public void WriteLine(string line)
    {
      AppendText(line + "\r\n");
      SelectionStart = Text.Length;
      ScrollToCaret();
    }
    public void WriteLine()
    {
      AppendText("\r\n");
      SelectionStart = Text.Length;
      ScrollToCaret();
    }
  }
  public class LogConsole : ILog
  {
    public void Write(string text)
    {
      Console.Write(text);
    }
    public void WriteLine(string line)
    {
      Console.WriteLine(line);
    }
    public void WriteLine()
    {
      Console.WriteLine();
    }
  }
  public class LogVoid : ILog
  {
    public void Write(string text)
    {
    }
    public void WriteLine(string line)
    {
    }
    public void WriteLine()
    {
    }
  }
  public class LogFile : ILog
  {
    StreamWriter _sw;

    public LogFile(string filename)
    {
      _sw = File.CreateText(filename);
    }

    public void Write(string text)
    {
      _sw.Write(text);
    }
    public void WriteLine(string line)
    {
      _sw.WriteLine(line);
    }
    public void WriteLine()
    {
      _sw.WriteLine();
    }
  }
}
