﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CVUT.Vorel
{
  class PrintText
  {
    private string _textToPrint = "";
    private PrintDocument _docToPrint = new PrintDocument();
    private PrintDialog _dialForPrint = new PrintDialog();
    private PrintPreviewDialog _dialForPrintPreview = new PrintPreviewDialog();
    private string _docName = "";
    bool _multipage = false;

    public PrintText() 
    {
    }
    public PrintText(string textToPrint, PrintDocument docToPrint, PrintDialog dialForPrint)
    {
      _textToPrint = textToPrint;
      _docToPrint = docToPrint;
      _dialForPrint = dialForPrint;
    }
    public PrintText(string textToPrint, PrintDialog dialForPrint, string docName)
    {
      _textToPrint = textToPrint;
      _docToPrint = docToPrint;
      _docName = docName;
    }

    public string textToPrint
    {
      get
      {
        return _textToPrint;
      }
      set
      {
        _textToPrint = value;
      }
    }
    public PrintDocument docToPrint
    {
      get
      {
        return _docToPrint;
      }
      set
      {
        _docToPrint = value;
      }
    }
    public PrintDialog dialForPrint
    {
      get
      {
        return _dialForPrint;
      }
      set
      {
        _dialForPrint = value;
      }
    }
    public PrintPreviewDialog dialForPrintPreview
    {
      get
      {
        return _dialForPrintPreview;
      }
      set
      {
        _dialForPrintPreview = value;
      }
    }

    public void Print(object sender, EventArgs e)
    {
      _docToPrint.DocumentName = _docName;
      // Allow the user to choose the page range he or she would like to print.
      _dialForPrint.AllowSomePages = true;

      // Show the help button.
      _dialForPrint.ShowHelp = true;

      // Set the Document property to the PrintDocument for which the PrintPage Event has been handled. To display the
      // dialog, either this property or the PrinterSettings property must be set 
      _dialForPrint.Document = _docToPrint;

      // Associate the PrintPage event handler with the PrintPage event.
      _docToPrint.PrintPage += new System.Drawing.Printing.PrintPageEventHandler(printDocument3_PrintPage);

      // If the result is OK then print the document.
      if (_dialForPrint.ShowDialog() == DialogResult.OK)
      {
        _multipage = false;
        _docToPrint.Print();
      }
    }
    private void printDocument1_PrintPage(object sender, PrintPageEventArgs e)
    {
      e.HasMorePages = _multipage;
      int charactersOnPage = 0;
      int linesPerPage = 0;
      //Font PrintFont = new Font("Microsoft Sans Serif", 9);
      //Font PrintFont = CVUT.Vorel.heat.controls.ucHomoHeat.DefaultFont;
      Font PrintFont = SystemFonts.DefaultFont;
      
      int TabIndent = 100; //1 inch is 72 points
      StringFormat printingStringFormat;

      printingStringFormat = (StringFormat)System.Drawing.StringFormat.GenericTypographic.Clone();

      // 100 should be enough for everyone ...err ?
      float[] tabStops = new float[100];
      for (int i = 0; i < tabStops.Length; ++i)
      {
        tabStops[i] = TabIndent;
      }

      printingStringFormat.SetTabStops(0, tabStops);

      // Sets the value of charactersOnPage to the number of characters 
      // of stringToPrint that will fit within the bounds of the page.
      e.Graphics.MeasureString(_textToPrint, PrintFont,
          e.MarginBounds.Size, printingStringFormat,
          out charactersOnPage, out linesPerPage);

      // Draws the string within the bounds of the page
      e.Graphics.DrawString(_textToPrint, PrintFont, Brushes.Black,
      e.MarginBounds, printingStringFormat);

      // Remove the portion of the string that has been printed.
      _textToPrint = _textToPrint.Substring(charactersOnPage);

      // Check to see if more pages are to be printed.
      _multipage = e.HasMorePages = (_textToPrint.Length > 0);
    }
    private void printDocument2_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs e)
    {
      Font objFont = new Font("Microsoft Sans Serif", 10F);//sets the font type and size
      float fTopMargin = e.MarginBounds.Top;
      float fLeftMargin = 50;//sets left margin
      float fRightMargin = e.MarginBounds.Right - 150;//sets right margin   
      string employee_id = String.Concat("EMPLOYEE ID:-  ", _textToPrint);//prints EMPLOYEE ID:- and text in combobox2
      string employee_name = String.Concat("EMPLOYEE NAME:-  ", _textToPrint);

      e.Graphics.DrawString(employee_id, objFont, Brushes.Black, fLeftMargin, fTopMargin);

      fTopMargin += objFont.GetHeight() * 2;//skip two lines

      e.Graphics.DrawString(employee_name, objFont, Brushes.Black, fLeftMargin, fTopMargin);

      objFont.Dispose();

      e.HasMorePages = false;
    }
    protected void printDocument3_PrintPage(object sender, System.Drawing.Printing.PrintPageEventArgs ev)
    {
      float linesPerPage = 0;
      float yPosition = 0;
      int count = 0;
      float leftMargin = ev.MarginBounds.Left;
      float topMargin = ev.MarginBounds.Top;
      string line = null;
      StringReader _myReader = new StringReader(_textToPrint);
      //Font printFont = this.richTextBox1.Font;
      Font printFont = SystemFonts.DefaultFont;
      SolidBrush myBrush = new SolidBrush(Color.Black);
      // Work out the number of lines per page, using the MarginBounds.
      linesPerPage = ev.MarginBounds.Height / printFont.GetHeight(ev.Graphics);
      // Iterate over the string using the StringReader, printing each line.
      while (count < linesPerPage && ((line = _myReader.ReadLine()) != null))
      {
        // calculate the next line position based on the height of the font according to the printing device
        yPosition = topMargin + (count * printFont.GetHeight(ev.Graphics));
        // draw the next line in the rich edit control
        ev.Graphics.DrawString(line, printFont, myBrush, leftMargin, yPosition, new StringFormat());
        count++;
      }
      // If there are more lines, print another page.
      if (line != null)
        ev.HasMorePages = true;
      else
        ev.HasMorePages = false;
      myBrush.Dispose();
    }

    public void printPreview(object sender, EventArgs e)
    {
      _dialForPrintPreview.Document = _docToPrint;
      // Associate the PrintPage event handler with the PrintPage event.
      _docToPrint.PrintPage += new System.Drawing.Printing.PrintPageEventHandler(printDocument1_PrintPage);
      _dialForPrintPreview.ShowDialog(); // Show the print preview dialog, uses print page event to draw preview screen
    }

  }
}
