﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CVUT.Vorel
{
  class ProblemSolving
  {
    public static bool WeaveCompositeElasticity(Tensor input, string type, TextWriter tw)
    {
      //Calculate stiffness matrix for plane weave composit (characterized in 'input') 
      //and write semi-axes of bundle and stiffness matrix elements into a file defined by 'tw'
      //not done for histograms

      if (input.which_order != 1 || input.how_many_lines != 19)
      {
        throw new Exception("WeaveCompositElasticity: Wrong input tensor!!!");
      }


      Tensor L = new Tensor(6, 6);
      elasticity.Matrix matrice = new elasticity.Matrix(input[0], input[1]);
      elasticity.Tow svazek = new elasticity.Tow();

      svazek.a = input[2];
      svazek.b = input[3];
      svazek.g = input[4];
      svazek.h = input[5];

      svazek.SemiAxes[0] = input[6];
      svazek.SemiAxes[1] = input[7];
      svazek.SemiAxes[2] = input[8];

      svazek.StiffnessMatrix[0, 0] = input[9];
      svazek.StiffnessMatrix[0, 1] = svazek.StiffnessMatrix[1, 0] = input[10];
      svazek.StiffnessMatrix[0, 2] = svazek.StiffnessMatrix[2, 0] = input[11];
      svazek.StiffnessMatrix[1, 1] = input[12];
      svazek.StiffnessMatrix[1, 2] = svazek.StiffnessMatrix[2, 1] = input[13];
      svazek.StiffnessMatrix[2, 2] = input[14];
      svazek.StiffnessMatrix[3, 3] = input[15];
      svazek.StiffnessMatrix[4, 4] = input[16];
      svazek.StiffnessMatrix[5, 5] = input[17];

      svazek.VolFrac = input[18];

      L = elasticity.MT.MT_routine(matrice, svazek, 1, type, true);


      // write a line of text to the file
      tw.WriteLine(Convert.ToString(svazek.SemiAxes[0], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(svazek.SemiAxes[1], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(svazek.SemiAxes[2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[0, 0], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[0, 1], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[0, 2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[1, 1], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[1, 2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[2, 2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[3, 3], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[4, 4], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(L[5, 5], Vorel.Constants.Separators.FormatProvider) + "\n");

      return (true);
    }

    public static bool WeaveCompositeHeat(Tensor input, string type, TextWriter tw)
    {
      //Calculate heat conductivity matrix for plane weave composit (characterized in 'input') 
      //and write semi-axes of bundle and stiffness matrix elements into a file defined by 'tw'
      //not done for histograms

      if (input.which_order != 1 || input.how_many_lines != 15)
      {
        throw new Exception("WeaveCompositElasticity: Wrong input tensor!!!");
      }


      Tensor K = new Tensor(3, 3);
      heat.Matrix matrice = new heat.Matrix(input[0]);
      heat.Tow svazek = new heat.Tow();

      svazek.a = input[1];
      svazek.b = input[2];
      svazek.g = input[3];
      svazek.h = input[4];

      svazek.SemiAxes[0] = input[5];
      svazek.SemiAxes[1] = input[6];
      svazek.SemiAxes[2] = input[7];

      svazek.HeatConducTensor[0, 0] = input[8];
      svazek.HeatConducTensor[0, 1] = svazek.HeatConducTensor[1, 0] = input[9];
      svazek.HeatConducTensor[0, 2] = svazek.HeatConducTensor[2, 0] = input[10];
      svazek.HeatConducTensor[1, 1] = input[11];
      svazek.HeatConducTensor[1, 2] = svazek.HeatConducTensor[2, 1] = input[12];
      svazek.HeatConducTensor[2, 2] = input[13];

      svazek.VolFrac = input[14];

      K =heat.MT.MT_routine(matrice, svazek, 1, type, true);



      // write a line of text to the file
      tw.WriteLine(Convert.ToString(svazek.SemiAxes[0], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(svazek.SemiAxes[1], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(svazek.SemiAxes[2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(K[0, 0], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(K[0, 1], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(K[0, 2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(K[1, 1], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(K[1, 2], Vorel.Constants.Separators.FormatProvider) + "\t" +
                   Convert.ToString(K[2, 2], Vorel.Constants.Separators.FormatProvider) + "\n");

      return (true);
    }

  }
}
