﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace CVUT.Vorel.weaveComposit.controls
{
  public partial class ucWeave : UserControl
  {
    private elasticity.Matrix _elasticityMatrix = new elasticity.Matrix();
    private elasticity.Tow _elasticityTow = new elasticity.Tow();
    private heat.Matrix _heatMatrix = new heat.Matrix();
    private heat.Tow _heatTow = new heat.Tow();
    private string _projectName = "";
    private string _typeOfHomogenization = "";
    Tensor L_result = new Tensor(6, 6);
    Tensor K_result = new Tensor(3, 3);
    ///////////////////////////////////////////////////////////////////////////////////

    public ucWeave()
    {
      InitializeComponent();
      rbElasticity.Checked = true;

      //initialization of matrix
      ucElasticityMaterialMatrix.MaterialConstituent = _elasticityMatrix;
      ucElasticityMaterialTow.MaterialConstituent = _elasticityTow;
      ucHeatMaterialMatrix.LoadK(_heatMatrix);
      ucHeatMaterialTow.LoadK(_heatTow);

      //initialization of tow
      LoadTow(_elasticityTow);
      LoadTow(_heatTow);

      tbxProjectName.Text = "New Project";
      dtbCi_ValueChanged(this, EventArgs.Empty);
      
      //homogenization type
      rbHarmonic.Checked = true;
      cbHomoKind.SelectedIndex = 0;

      //method
      cbAnalysis.SelectedIndex = 0;
    }

    public LogLabel LogLabel
    {
      get
      {
        return logLabel;
      }
    }
    public string ProjectName
    {
      get
      {
        return _projectName;
      }
      set
      {
        _projectName = value;
      }
    }
    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public elasticity.Matrix ElasticityMatrix
    {
      get
      {
        return _elasticityMatrix;
      }
      set
      {
        _elasticityMatrix = value;
        LoadMatrix(_elasticityMatrix);
      }
    }
    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public heat.Matrix HeatMatrix
    {
      get
      {
        return _heatMatrix;
      }
      set
      {
        _heatMatrix = value;
        LoadMatrix(_heatMatrix);
      }
    }
    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public elasticity.Tow ElasticityTow
    {
      get
      {
        return _elasticityTow;
      }
      set
      {
        _elasticityTow = value;
        LoadTow(_elasticityTow);
      }
    }
    [
      Browsable(false),
      DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)
    ]
    public heat.Tow HeatTow
    {
      get
      {
        return _heatTow;
      }
      set
      {
        _heatTow = value;
        LoadTow(_heatTow);
      }
    }
    ///////////////////////////////////////////////////////////////////////////////////
    private void dtbProjectName_TextChanged(object sender, EventArgs e)
    {
      _projectName = tbxProjectName.Text;
    }
    private void rbKind_CheckedChanged(object sender, EventArgs e)
    {
      if (rbElasticity.Checked)
      {
        ucElasticityMaterialMatrix.Visible = true;
        ucElasticityMaterialMatrix.Enabled = true;

        ucHeatMaterialMatrix.Visible = false;
        ucHeatMaterialMatrix.Enabled = false;

        ucElasticityMaterialTow.Visible = true;
        ucElasticityMaterialTow.Enabled = true;

        ucHeatMaterialTow.Visible = false;
        ucHeatMaterialTow.Enabled = false;
      }
      else if (rbHeat.Checked)
      {
        ucElasticityMaterialMatrix.Visible = false;
        ucElasticityMaterialMatrix.Enabled = false;

        ucHeatMaterialMatrix.Visible = true;
        ucHeatMaterialMatrix.Enabled = true;

        ucElasticityMaterialTow.Visible = false;
        ucElasticityMaterialTow.Enabled = false;

        ucHeatMaterialTow.Visible = true;
        ucHeatMaterialTow.Enabled = true;
      }
    }
    public void Change_rbKind(Constants.EProblemKind problem)
    {
      if (problem == Constants.EProblemKind.Elasticity)
      {
        rbElasticity.Checked = true;
      }
      else if (problem == Constants.EProblemKind.Heat)
      {
        rbHeat.Checked = true;
      }

      rbKind_CheckedChanged(this, EventArgs.Empty);
    }
    private void dtbCell_ValueChanged(object sender, EventArgs e)
    {
      _elasticityTow.a = dtbA.Value;
      _elasticityTow.b = dtbB.Value;
      _elasticityTow.g = dtbG.Value;
      _elasticityTow.h = dtbH.Value;

      _heatTow.a = dtbA.Value;
      _heatTow.b = dtbB.Value;
      _heatTow.g = dtbG.Value;
      _heatTow.h = dtbH.Value;
    }

    private void dtbCi_ValueChanged(object sender, EventArgs e)
    {
      double res;

      res = 1 - dtbCi.Value;
      lbC0.Text = res.ToString();

      if (res <= 0)
      {
        MessageBox.Show("Volume of tow is exceeding 1", "Homogenization", MessageBoxButtons.OK, MessageBoxIcon.Hand);
        lbC0.ForeColor = Color.Red;
        lbC0.Font = new Font(this.Font.Name, (this.Font.SizeInPoints + 5), FontStyle.Bold);
      }
      else if (res < 0.5)
      {
        MessageBox.Show("The computation isn't accurate for this volume of tow", "Homogenization", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
        lbC0.ForeColor = Color.Blue;
        lbC0.Font = new Font(this.Font.Name, (this.Font.SizeInPoints + 3), FontStyle.Bold);
      }
      else
      {
        lbC0.ForeColor = this.ForeColor;
        lbC0.Font = this.Font;
      }

      _elasticityTow.VolFrac = dtbCi.Value;
      _heatTow.VolFrac = dtbCi.Value;
    }
    private void rbSphere_CheckedChanged(object sender, EventArgs e)
    {
      Constants.EInclusionKind dummy = Constants.EInclusionKind.Empty;

      if (rbSphere.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = false;
        dtbA3.Enabled = false;

        dummy = Constants.EInclusionKind.Sphere;
      }
      else if (rbCylinder.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = false;
        dtbA3.Enabled = false;

        dummy = Constants.EInclusionKind.Cylinder;
      }
      else if (rbEllipticCylinder.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = true;
        dtbA3.Enabled = false;

        dummy = Constants.EInclusionKind.EllipticCylinder;
      }
      else if (rbPennyShape.Checked)
      {
        dtbA1.Enabled = false;
        dtbA2.Enabled = false;
        dtbA3.Enabled = true;

        if (dtbA3.Value > 1)
          dtbA3.Value = 1;

        dummy = Constants.EInclusionKind.PennyShape;
      }
      else if (rbEllipsoid.Checked)
      {
        dtbA1.Enabled = true;
        dtbA2.Enabled = true;
        dtbA3.Enabled = true;

        dummy = Constants.EInclusionKind.Ellipsoid;
      }

      switch (dummy)
      {
        case Constants.EInclusionKind.Sphere:
          dtbA3.Value = dtbA2.Value = dtbA1.Value;
          break;
        case Constants.EInclusionKind.Cylinder:
          dtbA2.Value = dtbA1.Value;
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.PennyShape:
          dtbA1.Value = dtbA2.Value = 1;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          break;
      }

      _elasticityTow.SemiAxes[0] = dtbA1.Value;
      _elasticityTow.SemiAxes[1] = dtbA2.Value;
      _elasticityTow.SemiAxes[2] = dtbA3.Value;

      _heatTow.SemiAxes[0] = dtbA1.Value;
      _heatTow.SemiAxes[1] = dtbA2.Value;
      _heatTow.SemiAxes[2] = dtbA3.Value;

    }
    private void dtbA_ValueChanged(object sender, EventArgs e)
    {
      Constants.EInclusionKind dummy = Constants.EInclusionKind.Empty;
      if (rbSphere.Checked)
      {
        dummy = Constants.EInclusionKind.Sphere;
      }
      else if (rbCylinder.Checked)
      {
        dummy = Constants.EInclusionKind.Cylinder;
      }
      else if (rbEllipticCylinder.Checked)
      {
        dummy = Constants.EInclusionKind.EllipticCylinder;
      }
      else if (rbPennyShape.Checked)
      {
        dummy = Constants.EInclusionKind.PennyShape;
      }
      else if (rbEllipsoid.Checked)
      {
        dummy = Constants.EInclusionKind.Ellipsoid;
      }


      switch (dummy)
      {
        case Constants.EInclusionKind.Sphere:
          dtbA2.Value = dtbA1.Value;
          dtbA3.Value = dtbA1.Value;
          break;
        case Constants.EInclusionKind.Cylinder:
          dtbA2.Value = dtbA1.Value;
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          dtbA3.Value = Double.PositiveInfinity;
          break;
        case Constants.EInclusionKind.PennyShape:
          dtbA1.Value = 1;
          dtbA2.Value = 1;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          break;
      }

      _elasticityTow.SemiAxes[0] = dtbA1.Value;
      _elasticityTow.SemiAxes[1] = dtbA2.Value;
      _elasticityTow.SemiAxes[2] = dtbA3.Value;

      _heatTow.SemiAxes[0] = dtbA1.Value;
      _heatTow.SemiAxes[1] = dtbA2.Value;
      _heatTow.SemiAxes[2] = dtbA3.Value;
    }
    private void dtbFi_ValueChanged(object sender, EventArgs e)
    {
      _elasticityTow.EulerAngles[0] = dtbFi.Value;
      _elasticityTow.EulerAngles[1] = dtbTheta.Value;
      _elasticityTow.EulerAngles[2] = dtbBeta.Value;

      _heatTow.EulerAngles[0] = dtbFi.Value;
      _heatTow.EulerAngles[1] = dtbTheta.Value;
      _heatTow.EulerAngles[2] = dtbBeta.Value;
    }

    private void rbHomogenizationType_CheckedChanged(object sender, EventArgs e)
    {
      if (rbHarmonic.Checked)
      {
        btnHistogram.Visible = false;
        btnHistogram.Enabled = false;
      }
      else if (rbHistogram.Checked)
      {
        btnHistogram.Visible = true;
        btnHistogram.Enabled = true;
      }
    }
    private void btnHistogram_Click(object sender, EventArgs e)
    {
      if (openFileDialog1.ShowDialog() == DialogResult.OK)
      {
        try
        {
          _elasticityTow.histogram = new DataXY(openFileDialog1.FileName);
          _heatTow.histogram = new DataXY(_elasticityTow.histogram);

          if (_elasticityTow.histogram.Count == 0)
          {
            logLabel.WriteLine("NONE histogram data loaded");
          }
          else
          {
            logLabel.WriteLine("Histogram data loaded - " + _elasticityTow.histogram.Count.ToString() + " x 2");
          }
        }
        catch (Exception exc)
        {
          logLabel.WriteLine(exc.Message);
        }
      }
    }

    private void LoadMatrix(elasticity.Matrix elasticityMatrix)
    {
      ucElasticityMaterialMatrix.MaterialConstituent = elasticityMatrix;
    }
    public void LoadMatrix(elasticity.Constituent elasticityConstituent)
    {
      _elasticityMatrix.StiffnessMatrix = elasticityConstituent.StiffnessMatrix;
      LoadMatrix(_elasticityMatrix);
    }
    private void LoadMatrix(heat.Matrix heatMatrix)
    {
      ucHeatMaterialMatrix.LoadK(heatMatrix);
    }
    public void LoadMatrix(heat.Constituent heatConstituent)
    {
      _heatMatrix.HeatConducTensor = heatConstituent.HeatConducTensor;
      LoadMatrix(_heatMatrix);
    }
    private void LoadTow(elasticity.Tow elasticityTow)
    {
      _elasticityTow = elasticityTow;

      dtbA.Value = _elasticityTow.a;
      dtbB.Value = _elasticityTow.b;
      dtbG.Value = _elasticityTow.g;
      dtbH.Value = _elasticityTow.h;

      dtbA1.Value = _elasticityTow.SemiAxes[0];
      dtbA2.Value = _elasticityTow.SemiAxes[1];
      dtbA3.Value = _elasticityTow.SemiAxes[2];

      dtbFi.Value = _elasticityTow.EulerAngles[0];
      dtbTheta.Value = _elasticityTow.EulerAngles[1];
      dtbBeta.Value = _elasticityTow.EulerAngles[2];

      ucElasticityMaterialTow.MaterialConstituent = _elasticityTow;
      dtbCi.Value = _elasticityTow.VolFrac;
      dtbCi_ValueChanged(this, EventArgs.Empty);

      switch (_elasticityTow.InclusionKind)
      {
        case Constants.EInclusionKind.Sphere:
          rbSphere.Checked = true;
          break;
        case Constants.EInclusionKind.Cylinder:
          rbCylinder.Checked = true;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          rbEllipticCylinder.Checked = true;
          break;
        case Constants.EInclusionKind.PennyShape:
          rbPennyShape.Checked = true;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          rbEllipsoid.Checked = true;
          break;
      }

      // harmonization of heat and elasticity tows
      _heatTow.a = _elasticityTow.a;
      _heatTow.b = _elasticityTow.b;
      _heatTow.g = _elasticityTow.g;
      _heatTow.h = _elasticityTow.h;

      _heatTow.SemiAxes[0] = _elasticityTow.SemiAxes[0];
      _heatTow.SemiAxes[1] = _elasticityTow.SemiAxes[1];
      _heatTow.SemiAxes[2] = _elasticityTow.SemiAxes[2];

      _heatTow.EulerAngles[0] = _elasticityTow.EulerAngles[0];
      _heatTow.EulerAngles[1] = _elasticityTow.EulerAngles[1];
      _heatTow.EulerAngles[2] = _elasticityTow.EulerAngles[2];

      _heatTow.VolFrac = _elasticityTow.VolFrac;

    }
    public void LoadTow(elasticity.Inclusion elasticityInclusion)
    {
      elasticity.Tow dummy = new elasticity.Tow(_elasticityTow.a, _elasticityTow.b, _elasticityTow.g, 
                                                _elasticityTow.h, elasticityInclusion);
      LoadTow(dummy);
    }
    public void LoadTow(elasticity.Constituent elasticityConstituent)
    {
      _elasticityTow.StiffnessMatrix = elasticityConstituent.StiffnessMatrix;
      LoadTow(_elasticityTow);
    }
    private void LoadTow(heat.Tow heatTow)
    {
      _heatTow = heatTow;

      dtbA.Value = _heatTow.a;
      dtbB.Value = _heatTow.b;
      dtbG.Value = _heatTow.g;
      dtbH.Value = _heatTow.h;

      dtbA1.Value = _heatTow.SemiAxes[0];
      dtbA2.Value = _heatTow.SemiAxes[1];
      dtbA3.Value = _heatTow.SemiAxes[2];

      dtbFi.Value = _heatTow.EulerAngles[0];
      dtbTheta.Value = _heatTow.EulerAngles[1];
      dtbBeta.Value = _heatTow.EulerAngles[2];

      ucHeatMaterialTow.LoadK(_heatTow);
      dtbCi.Value = _heatTow.VolFrac;
      dtbCi_ValueChanged(this, EventArgs.Empty);

      switch (_heatTow.InclusionKind)
      {
        case Constants.EInclusionKind.Sphere:
          rbSphere.Checked = true;
          break;
        case Constants.EInclusionKind.Cylinder:
          rbCylinder.Checked = true;
          break;
        case Constants.EInclusionKind.EllipticCylinder:
          rbEllipticCylinder.Checked = true;
          break;
        case Constants.EInclusionKind.PennyShape:
          rbPennyShape.Checked = true;
          break;
        case Constants.EInclusionKind.Ellipsoid:
          rbEllipsoid.Checked = true;
          break;
      }

      // harmonization of heat and elasticity tows
      _elasticityTow.a = _heatTow.a;
      _elasticityTow.b = _heatTow.b;
      _elasticityTow.g = _heatTow.g;
      _elasticityTow.h = _heatTow.h;

      _elasticityTow.SemiAxes[0] = _heatTow.SemiAxes[0];
      _elasticityTow.SemiAxes[1] = _heatTow.SemiAxes[1];
      _elasticityTow.SemiAxes[2] = _heatTow.SemiAxes[2];

      _elasticityTow.EulerAngles[0] = _heatTow.EulerAngles[0];
      _elasticityTow.EulerAngles[1] = _heatTow.EulerAngles[1];
      _elasticityTow.EulerAngles[2] = _heatTow.EulerAngles[2];

      _elasticityTow.VolFrac = _heatTow.VolFrac;
    }
    public void LoadTow(heat.Constituent heatConstituent)
    {
      _heatTow.HeatConducTensor = heatConstituent.HeatConducTensor;
      LoadTow(_heatTow);
    }
    public void LoadTow(heat.Inclusion heatInclusion)
    {
      heat.Tow dummy = new heat.Tow(_heatTow.a, _heatTow.b, _heatTow.g,
                                                _heatTow.h, heatInclusion);
      LoadTow(dummy);
    }

    private void btnCalculate_Click(object sender, EventArgs e)
    {
      switch (cbHomoKind.SelectedIndex)
      {
        case 0:
          _typeOfHomogenization = "twoPhase";
          break;

        case 1:
          _typeOfHomogenization = "threePhase";
          break;

        default:
          throw new Exception("Weave: controls: btnCalculate_Click: wrong case!!!");
      }

      if (rbElasticity.Checked)
      {
        try
        {
          if (rbHarmonic.Checked)
          {
            L_result = elasticity.MT.MT_routine(_elasticityMatrix, _elasticityTow, 1, _typeOfHomogenization, true);
          }
          else
          {
            L_result = elasticity.MT.MT_routine(_elasticityMatrix, _elasticityTow, 1, _typeOfHomogenization, false);
          }
          

          //output
          logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
          logLabel.WriteLine(_projectName);
          logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
          logLabel.WriteLine("================= INPUT =================");
          logLabel.WriteLine("--- Matrix ---");
          logLabel.WriteLine("---------------------------");
          _elasticityMatrix.StiffnessMatrix.InverseMatrix().Print("Compliance matrix", logLabel, false);
          _elasticityMatrix.StiffnessMatrix.Print("Stiffness matrix", logLabel, false);

          logLabel.WriteLine("--- Unit Cell ---");
          logLabel.WriteLine("---------------------------");
          logLabel.WriteLine("a = " + _heatTow.a.ToString());
          logLabel.WriteLine("b = " + _heatTow.b.ToString());
          logLabel.WriteLine("g = " + _heatTow.g.ToString());
          logLabel.WriteLine("h = " + _heatTow.h.ToString());

          logLabel.WriteLine("--- Tow ---");
          logLabel.WriteLine("---------------------------");
          logLabel.WriteLine("ci = " + _heatTow.VolFrac.ToString());
          _elasticityTow.SemiAxes.Print("Semi-axis", logLabel, false);
          _elasticityTow.EulerAngles.Print("Euler angles", logLabel, false);
          _elasticityTow.StiffnessMatrix.InverseMatrix().Print("Compliance matrix", logLabel, false);
          _elasticityTow.StiffnessMatrix.Print("Stiffness matrix", logLabel, false);

          logLabel.WriteLine("================ OUTPUT ================");
          L_result.InverseMatrix().Print("Effective compliance matrix", logLabel, false);
          L_result.Print("Effective stiffness matrix", logLabel, false);
        }
        catch (Exception exc)
        {
          logLabel.WriteLine(exc.Message);
        }
      }
      else if (rbHeat.Checked)
      {
        try
        {
          if (rbHarmonic.Checked)
          {
            K_result = heat.MT.MT_routine(_heatMatrix, _heatTow, 1, _typeOfHomogenization, true);
          }
          else
          {
            K_result = heat.MT.MT_routine(_heatMatrix, _heatTow, 1, _typeOfHomogenization, false);
          }

          //output
          logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
          logLabel.WriteLine(_projectName);
          logLabel.WriteLine("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%");
          logLabel.WriteLine("================= INPUT =================");
          logLabel.WriteLine("--- Matrix ---");
          logLabel.WriteLine("---------------------------");
          _heatMatrix.HeatConducTensor.Print("Conductivity", logLabel, false);

          logLabel.WriteLine("--- Unit Cell ---");
          logLabel.WriteLine("---------------------------");
          logLabel.WriteLine("a = " + _heatTow.a.ToString());
          logLabel.WriteLine("b = " + _heatTow.b.ToString());
          logLabel.WriteLine("g = " + _heatTow.g.ToString());
          logLabel.WriteLine("h = " + _heatTow.h.ToString());

          logLabel.WriteLine("--- Tow ---");
          logLabel.WriteLine("---------------------------");
          logLabel.WriteLine("ci = " + _heatTow.VolFrac.ToString());
          _heatTow.SemiAxes.Print("Semi-axis", logLabel, false);
          _heatTow.EulerAngles.Print("Euler angles", logLabel, false);
          _heatTow.HeatConducTensor.Print("Conductivity", logLabel, false);

          logLabel.WriteLine("================ OUTPUT ================");
          K_result.Print("Effective conductivity", logLabel, false);
        }
        catch (Exception exc)
        {
          logLabel.WriteLine(exc.Message);
        }
      }
    }
    private void btnSave_Click(object sender, EventArgs e)
    {
      if (saveFileDialog1.ShowDialog() == DialogResult.OK)
      {
        using (StreamWriter sw = File.CreateText(saveFileDialog1.FileName))
        {
          sw.Write(logLabel.Text);
        }
      }
    }
    public void btnPrint_Click(object sender, EventArgs e)
    {
      PrintText _text = new PrintText(logLabel.Text, printDialog1, _projectName);

      _text.Print(sender, e);
    }
    private void btnClear_Click(object sender, EventArgs e)
    {
      logLabel.Clear();
    }
    ///////////////////////////////////////////////////////////////////////////////////
    private void clearProject(object sender, EventArgs e)
    {
      // matrice
      _elasticityMatrix.ClearMatrix();
      LoadMatrix(_elasticityMatrix);
      _heatMatrix.ClearMatrix();
      LoadMatrix(_heatMatrix);
      
      // tow
      _elasticityTow.ClearTow();
      LoadTow(_elasticityTow);
      _heatTow.ClearTow();
      LoadTow(_heatTow);

      tbxProjectName.Text = "New Project";
    }
    public void newProject(object sender, EventArgs e)
    {
      try
      {
        clearProject(sender, e);
      }
      catch (Exception exc)
      {
        logLabel.WriteLine(exc.Message);
      }
    }

    public void saveProject(string fileName)
    {
      if (rbElasticity.Checked)
      {
        CVUT.Vorel.Save.FileToSave(_elasticityMatrix, _elasticityTow, ProjectName, fileName);
      }
      else if (rbHeat.Checked)
      {
        CVUT.Vorel.Save.FileToSave(_heatMatrix, _heatTow, ProjectName, fileName);
      }
    }
    public void openProject(object sender, EventArgs e, string fileName)
    {
      try
      {
        clearProject(sender, e);

        switch(CVUT.Vorel.Load.WhichCase(fileName))
        {
          case Constants.EProblemKind.Elasticity:
            CVUT.Vorel.Load.LoadToProject(out _elasticityMatrix, out _elasticityTow, out _projectName, fileName);

            //matrix
            if (_elasticityMatrix.MaterialKind == Constants.EMaterialKind.FullyAnisotropic)
            {
              throw new Exception("weaveComposite: ucWeave: openProject: Matrix has fully anisotropy!!!");
            }
            LoadMatrix(_elasticityMatrix);

            //tow
            if (_elasticityTow.MaterialKind == Constants.EMaterialKind.FullyAnisotropic)
            {
              throw new Exception("weaveComposite: ucWeave: openProject: openProject: Tow has fully anisotropy!!!");
            }
            LoadTow(_elasticityTow);

            Change_rbKind(Constants.EProblemKind.Elasticity);

            break;

          case Constants.EProblemKind.Heat:
            CVUT.Vorel.Load.LoadToProject(out _heatMatrix, out _heatTow, out _projectName, fileName);

            //matrix
            if (_heatMatrix.MaterialKind == Constants.EMaterialKind.FullyAnisotropic)
            {
              throw new Exception("weaveComposite: ucWeave: openProject: Matrix has fully anisotropy!!!");
            }
            LoadMatrix(_heatMatrix);

            //tow
            if (_heatTow.MaterialKind == Constants.EMaterialKind.FullyAnisotropic)
            {
              throw new Exception("weaveComposite: ucWeave: openProject: openProject: Tow has fully anisotropy!!!");
            }
            LoadTow(_heatTow);

            Change_rbKind(Constants.EProblemKind.Heat);

            break;

          default:
            throw new Exception("weaveComposite: ucWeave: openProject: wrong problem kind!!!");
        }

        tbxProjectName.Text = _projectName;
      }
      catch (Exception exc)
      {
        logLabel.WriteLine(exc.Message);
      }
    }

    private void ucElasticityMaterialTow_StiffChanged(object sender, EventArgs e)
    {
      _elasticityTow.StiffnessMatrix = ucElasticityMaterialTow.MaterialConstituent.StiffnessMatrix;
    }

  }
}
